"CatWalk - model browser for TurboGears"
__version__ = "0.9.4"
__author__ = "Ronald Jaramillo"
__email__ = "ronald@checkandhsare.com"
__copyright__ = "Copyright 2005 Ronald Jaramillo"
__license__ = "MIT"
import cPickle as pickle
import cherrypy

import turbogears
from turbogears import identity
import sqlobject
from sqlobject.sqlbuilder import *
import datetime
import pkg_resources
import time
import os
import struct
import socket
from browse import Browse
import re,datetime

date_parser = re.compile(r"""^
    (?P<year>\d{4,4})
    (?:
        -
        (?P<month>\d{1,2})
        (?:
            -
            (?P<day>\d{1,2})
            (?:
                T
                (?P<hour>\d{1,2})
                :
                (?P<minute>\d{1,2})
                (?:
                    :
                    (?P<second>\d{1,2})
                    (?:
                        \.
                        (?P<dec_second>\d+)?
                    )?
                )?                                        
                (?:
                    Z
                    |
                    (?:
                        (?P<tz_sign>[+-])
                        (?P<tz_hour>\d{1,2})
                        :
                        (?P<tz_min>\d{2,2})
                    )
                )
            )?
        )?
    )?
$""", re.VERBOSE)

def parse_datetime(s):
    """ parse a string and return a datetime object. """
    assert isinstance(s, basestring)
    r = date_parser.search(s)
    try:
        a = r.groupdict('0')
    except:
        raise ValueError, 'invalid date string format'

    dt = datetime.datetime(int(a['year']),
                           int(a['month']) or 1,
                           int(a['day']) or 1,
                           # If not given these will default to 00:00:00.0
                           int(a['hour']),
                           int(a['minute']),
                           int(a['second']),
                           # Convert into microseconds
                           int(a['dec_second'])*100000,
                           )
    tz_hours_offset = int(a['tz_hour'])
    tz_mins_offset = int(a['tz_min'])
    if a.get('tz_sign', '+') == "-":
        return dt + datetime.timedelta(hours = tz_hours_offset,
                                       minutes = tz_mins_offset)
    else:
        return dt - datetime.timedelta(hours = tz_hours_offset,
                                       minutes = tz_mins_offset)




class CatWalk(turbogears.controllers.Controller):
    """Model Browser. Administration tool for listing, creating, updating or deleting your SQLObject instances"""
    __label__ ="CatWalk"
    __version__ = "0.9"
    __author__ = "Ronald Jaramillo"
    __email__ = "ronald@checkandshare.com"
    __copyright__ = "Copyright 2005 Ronald Jaramillo"
    __license__ = "MIT"
    browse = Browse()
    need_project = True
    icon = "/tg_static/images/catwalk.png"

    def __init__(self,model=None):
        """CatWalks initializer

           @param model: reference to a project model module
           @type model: yourproject.model 
        """
         
        if model:
            self.model = model
        else:
            self.model = turbogears.util.get_model()
        self.browse.catwalk = self 

        try:
            try:
                self._connection = model.hub
            except AttributeError:
                self._connection = sqlobject.sqlhub
        except:
            import traceback, sys
            traceback.print_exc(sys.exc_info())
            print """
                     CatWalk fail to load your model file,
                     If you are mounting CatWalk to your controller remember 
                     to import your model and pass a reference to it
                  """
        turbogears.config.update({'log_debug_info_filter.on':False})
        self.register_static_directory()

    def error(self,msg=''):
        """Generic error handler for json replies"""
        return dict(error=msg)
    error = turbogears.expose(format="json")(error)

    def load_object(self,object_name):
        """Return a class reference from the models module by name 
           
           @param object_name: name of the object 
           @type object_name: string
        """
        try:
            object = getattr(self.model,object_name)
        except: 
            msg='Fail to get reference to object %s'% object_name
            raise cherrypy.HTTPRedirect(turbogears.url('error', msg=msg))
        return object

    def load_instance(self,object_name,id):
        """Return and instance of the named object with the requested id"""
        object = self.load_object(object_name)          
        try:
            return object.get(id)                    
        except:
            msg='Fail to get instance of object:%s with id:%s'% (object_name,id)
            raise cherrypy.HTTPRedirect(turbogears.url('error', msg=msg))


    def object_field(self,row,column):
        """Return a dict containing the column name and value for 
           the specific column and row
           
           @param row: model instance
           @param column: dict containing columnName, title, type, 
                       eventually join, joinMethodName and/or options
           @type column: dict
        """
        if column.get('type','') == 'SOSingleJoin':
            try:
                subject =  getattr(row,column['joinMethodName'])
                value = '%s'% getattr(subject,column['labelColumn'])
            except:
                return {'column':column['columnName'],'value':'None','id':'0'}
            return {'column':column['columnName'],'value':value }           
        elif column.get('type','') in ('SORelatedJoin', 'SOSQLRelatedJoin'):
            return self.related_join_count(row,column)
        elif column.get('type','') in ('SOMultipleJoin', 'SOSQLMultipleJoin'):
            return self.multiple_join_count(row,column)
        elif column.get('type','') == 'SOForeignKey':
            return self.object_field_for_foreign_key(row,column)
        elif column.get('type','') == 'SOStringCol':
            value = getattr(row,column['columnName'])
            value = self.encode_label(value)
            return {'column':column['columnName'],'value':value }
        else:
            try:
                value = u'%s'% getattr(row,column['columnName'])
            except UnicodeDecodeError, e:
                value = unicode( getattr(row,column['columnName']),'UTF-8')
                
            return {'column':column['columnName'],'value':value }
    
    def multiple_join_count(self,row,column):
        """Return the total number of related objects"""
        try:
            columnObject = getattr( self.model, column['join'] )
            for clm in columnObject.sqlmeta.columnList:
                 if type(clm) == sqlobject.SOForeignKey:
                     if column['objectName'] == clm.foreignKey: fk = clm
            fkName = fk.name
            fkQuery = getattr( columnObject.q, str( fkName ) )
            select = columnObject.select( fkQuery == row.id)
            value = '%s' % select.count()
        except Exception, e:
            value='0'
        return { 'column':column['joinMethodName'], 'value':value }

    def related_join_count(self,row,column):
        """Return the total number of related objects"""
        try:
            value='%s'% len(list(getattr(row,column['joinMethodName'])))
        except:
            value='0'
        return { 'column':column['joinMethodName'], 'value':value }

    def object_field_for_foreign_key(self,row,column):
        """Return the foreign key value"""
        try:
            name = getattr(row,column['columnName'])
            value = getattr(name,column['labelColumn'])
            value = self.encode_label(value)
        except AttributeError:
            return {'column':column['columnName'],'value':'None','id':'0'} 

        return {'column':column['columnName'],
                'value':value,
                'id': '%s' % name.id 
               }

    def update_object(self,object_name,id,values):
        object = self.load_object(object_name)          
        instance = self.load_instance(object_name,id)
        columns = self.object_columns(object_name)        
        parameters = self.extract_parameters(columns,values)   
        instance.set(**parameters)

    def remove_related_joins_if_any(self,object_name,id):
        """Remove any related joins the object may have"""
        object = self.load_object(object_name)
        instance = self.load_instance(object_name,id)
        for join in object.sqlmeta.joins:
            if self.get_column_type(join) !='SORelatedJoin':continue 
            for join_reference in list(getattr(instance,join.joinMethodName)):
                method = self.uppercase_related_join_method(join.otherClassName,'remove')
                getattr(instance,method)(join_reference)

    def remove_object(self,object_name,id):
        """Remove the object by id"""
        object = self.load_object(object_name)
        self.remove_related_joins_if_any(object_name,id)

        try:
            object.delete(id)
        except:
            msg='Fail to delete instance id:%s for object:%s'% (id,object_name)
            raise cherrypy.HTTPRedirect(turbogears.url('error', msg=msg))

    def extract_parameters(self,cols,values):
        """Loop trough the columns and extract the values from the dictionary
           
           @param cols: column list 
           @param values: dict of submited values 
        """
        params = {}
        for col in cols:
            column_name = col['columnName']
            if values.has_key(column_name): 
                if col['type'] == 'SODateTimeCol': 
                    dt = values[column_name]
                    try: 
                        b = parse_datetime('%sZ'% dt.replace(' ','T'))
                    except: b  = None
                    values[column_name] = b

                if col['type'] == 'SOBoolCol':
                    try:b = bool(int(values[column_name]))
                    except:b = False
                    values[column_name] = b

                if col['type'] == 'SOFloatCol':
                    try:b = float(values[column_name])
                    except:b = 0.0 
                    values[column_name] = b

                if col['type'] == 'SOIntCol':
                    try:b = int(values[column_name])
                    except:b = 0 
                    values[column_name] = b

                if col['type'] == 'SOForeignKey':
                    self.extract_foreign_key(values,column_name)

                if col['type'] in ('SODecimalCol','SOCurrencyCol'):
                    self.extract_decimal_value(values,column_name)

                if values.has_key(column_name):
                    params[column_name] = values[column_name]
        return params

    def extract_foreign_key(self,values,column_name):
        if values[column_name] =='__default_none__':
            values[column_name]= None 
        else:
            try:b = int(values[column_name])
            except:b = 0 
            values[column_name] = b

    def extract_decimal_value(self,values,column_name):
        try:
            import decimal
            try:b = decimal.Decimal(values[column_name])
            except:b = decimal.Decimal('0.0') 
        except:
            try:b = float(values[column_name])
            except:b = 0.0 
        values[column_name] = b

    def object_instances(self,object_name,start=0):
        """return a dictionary containing all instances for the requested object
           
           @param object_name: name of the object 
           @type object_name: string
        """
        obj = self.load_object(object_name)
        total = 0
        page_size = 10
        start = int(start)
        try:
            query = obj.select()
            total = query.count()
            results = query[start:start+page_size]
            headers, rows = self.headers_and_rows(object_name,list(results))
        except Exception, e:
            msg='Fail to load object instance: %s'% e
            raise cherrypy.HTTPRedirect(turbogears.url('error', msg=msg))

        return dict(
                    objectName=object_name,
                    rows=rows,
                    headers=headers,
                    start=start,
                    page_size=page_size,
                    total=total,
                    hidden_columns=self.load_columns_visibility_state(object_name)
                   )

    def foreign_key_alternatives(self,foreign_key,column_label):
        """Return a list of dictionaries containing the posible foreignKey values
           
           @param foreign_key: name of the foreignKey object
           @type foreign_key: string
           @param column_label: name of the column to use as instance identifier
           @type column_label: string
        """
        obj = self.load_object(foreign_key)
        alt = []
        for x in list(obj.select()):
            label = self.encode_label(getattr(x,column_label))
            alt.append({'id':type(x.id)(x.id),'label':label})
        return alt

    def encode_label(self,label):
        try:
            return unicode(label,'UTF-8')
        except TypeError, e:
            return u'%s'% label  #this is an integer (fx. an id)
        except UnicodeEncodeError, e:
            return u'%s'% label 

    def headers_and_rows(self,objectName,rows):
        """Returns a tuple containing a list of rows and header labels
           
           @param objectName: name of the object 
           @type objectName: string
           @param rows: list of intances 
        """
        cols = self.object_columns(objectName) 
        labels = [{'column':col['columnName'],
                   'label':(col['title'] or col['columnName'])} for col in cols]
        labels.insert(0,{'column':'id','label':'ID'})
        values = []
        for row in rows:
            tmp = []
            tmp.append( self.object_field(row,{'columnName':'id'}))
            for col in cols:
                col['objectName']=objectName
                tmp.append( self.object_field(row,col))
            values.append(list(tmp))
        return labels,values

    def object_joins(self,objectName,id,join,joinType,joinObjectName=''):
        """Collect the joined instances into a dictionary

           @param objectName: name of the object 
           @type objectName: string
           @param id: id of the instance
           @type id: string
           @param join: name of the join (joinMethodName in SQLObject parlance)
           @type join: string
           @param joinType: name of join type
           @type joinType: string
           @param joinObjectName: otherClassName (in SQLObject parlance)
           @type joinObjectName: string
        """
        hostObject = objectName
        obj = self.load_object(objectName)
        try:
            rows = list(getattr(obj.get(id),join))
        except:
            msg='Error, joins objectName: %s, id: %s, join: %s'% (objectName,id,join)
            raise cherrypy.HTTPRedirect(turbogears.url('error', msg=msg))
        
        view = '%s_%s'% (hostObject,joinObjectName)
        hidden_columns = self.load_columns_visibility_state(view)

        joinsDict = dict(objectName=objectName,
                         rows=[],
                         headers=[],
                         join=join,
                         id=id,
                         joinType=joinType,
                         joinObjectName=joinObjectName,
                         hostObject=hostObject,
                         hidden_columns=hidden_columns)
        if not rows:
            return joinsDict

        c = '%s'% rows[0].sqlmeta.soClass
        objectName= c.split('.')[-1].replace("'>",'')
        headers, rows = self.headers_and_rows(objectName,rows)
        joinsDict['objectName'] = objectName 
        joinsDict['rows'] = rows
        joinsDict['headers'] = headers
        return joinsDict

    def object_representation(self,object):
        """Utility method that returns a stripped object representation"""
        repr = '%s'% object
        repr = repr.replace('<','')
        repr = repr.replace('>','')
        return repr

    def get_column_type(self,column):
        """Given a column representation return the column type"""
        column_type = '%r'% column
        return column_type.split()[0][1:].split('.')[-1]

    def column_title(self,column):
        if isinstance(column, sqlobject.col.SOForeignKey):
            return column.name.replace('ID','') 

        try: title =( getattr(column,'title') or '')
        except AttributeError: title =''
        return title

    def column_default(self,column,column_type):
        try: default = column.default
        except: return ''
        if default== sqlobject.sqlbuilder.NoDefault:return ''
        if column_type in ('SOIntCol',
                           'SOFloatCol',
                           'SOStringCol',
                           'SODecimalCol',
                           'SOCurrencyCol',
                           'SOBoolCol'):return default
        if column_type == 'SODateTimeCol':
            d = '%s'% default
            return ':'.join( d.split(':')[:-1] )
        if column_type == 'SODateCol':
            d = '%s'% default
            return d.split()[0] 
        return ''

    def get_column_properties(self,column_name,column):
        """Return a dictionary containing the column properties
           Depending on the column type the properties returned could be:
           type, title, join (otherClassName), joinMethodName, 
           length, varchar,labelColumn,opetions

           @param column_name: name of the column 
           @type column_name: string
           @param column: column instance
        """
        props = {'type':self.get_column_type(column), 'columnName':column_name }
        props['title'] = self.column_title(column)
        props['default'] = self.column_default(column,props['type'])

        if props['type'] == 'SOEnumCol': 
            props['options'] = column.enumValues
        if props['type'] in ('SOMultipleJoin', 'SOSQLMultipleJoin',
          'SORelatedJoin', 'SOSQLRelatedJoin'):
            props['join'] = column.otherClassName
            props['joinMethodName'] = column.joinMethodName
        if props['type'] == 'SOSingleJoin':
            props['join'] = column.otherClassName
            props['joinMethodName'] = column.joinMethodName
            props['labelColumn'] = self.load_label_column_for_object(column.otherClassName)
            props['options'] = self.foreign_key_alternatives(column.otherClassName,
                                                                props['labelColumn'])
	    props['objectName'] = column.soClass.__name__
        if props['type'] in ['SOStringCol','SOUnicodeCol']:
            props = self.get_string_properties(column,props)
        if props['type'] == 'SOForeignKey':
            props = self.get_foreign_key_properties(column,props)
        return props
    
    def get_string_properties(self,column,properties):
        """Extract the SOStringCol properties from the column object"""
        properties['length'] = column.length
        properties['varchar'] = column.varchar
        return properties

    def get_foreign_key_properties(self,column,properties):
        """Extract the foreignKey properties from the column object"""
        properties['join'] = column.foreignKey
        properties['columnName'] = column.foreignName 
        properties['labelColumn'] = self.load_label_column_for_object(column.foreignKey)
        properties['options'] = self.foreign_key_alternatives(column.foreignKey,
                                                               properties['labelColumn'])
        if not column.notNone:
            properties['options'].insert(0,{'id':'__default_none__','label':'None'})
        return properties

    def object_columns(self,object_name):
        """Return list of columns properties arranged in dicts 
           @param object_name: name of the object 
           @type object_name: string
        """
        object = self.load_object(object_name)          
        cols = self.get_columns_for_object(object)
        return self.order_columns(object_name,cols)
    
    def get_columns_for_object(self,object):
        """Return list of columns properties arranged in dicts 
           @param object: model instance 
        """
        cols = []
        """get normal columns"""
        for column_name in object.sqlmeta.columns:
            column = object.sqlmeta.columns[column_name]
            if object._inheritable and column_name == 'childName':continue
            cols.append(self.get_column_properties(column_name,column)) 

        """get join columns"""
        for column in object.sqlmeta.joins:
            cols.append(self.get_column_properties(column.joinMethodName,column)) 

        """get inherited columns"""
        if object._inheritable and not self.is_inheritable_base_class(object):
            inherited_columns = self.get_inherited_columns(object)
            if inherited_columns: cols.extend(inherited_columns)
        return cols

    def get_inherited_columns(self,object):
        """Return the columns inherited from the parent class"""
        return self.get_columns_for_object(object._parentClass)

    def is_inheritable_base_class(self,obj):
        '''Check if the object is  a direct subclass of InheritableSQLObject'''
        return 'sqlobject.inheritance.InheritableSQLObject' in '%s'% (obj.__bases__,)

    """ ===================== CONFIGURATION STATE ============================"""
    def state_path(self):
        """Return the path to the catwalk session pickle -create a session directory if nescesary"""
        catwalk_session_dir = os.path.join(turbogears.util.get_package_name(),'catwalk-session')
        catwalk_session_dir = os.path.abspath( catwalk_session_dir )
        if not os.path.exists(catwalk_session_dir):
            try:
                os.mkdir(catwalk_session_dir)
            except IOError, e:
                msg='Fail to create session directory %s'% e
                raise cherrypy.HTTPRedirect(turbogears.url('error', msg=msg))
        return os.path.join(catwalk_session_dir,'session.pkl')

    def load_state(self):
        """Retrieve the pickled state from disc"""
        if not os.path.exists(self.state_path()): return {}
        try:
            return pickle.load(open(self.state_path(),'rb'))
        except PicklingError, e:
            msg='Fail to load pickled session file %s'% e
            raise cherrypy.HTTPRedirect(turbogears.url('error', msg=msg))

    def save_state(self,state):
        """Pickle the state"""
        try:
            pickle.dump(state,open(self.state_path(),'wb'),True)
        except PicklingError, e:
            msg='Fail to store pickled session file %s'% e
            raise cherrypy.HTTPRedirect(turbogears.url('error', msg=msg))

    def hide_columns(self,view,columns=''):
        state = self.load_state()
        hidden_columns = state.get('hidden_columns',{})    
        hidden_columns[view] =columns.split('|')
        print '*'*80
        print hidden_columns
        print '*'*80
        state['hidden_columns'] = hidden_columns
        self.save_state(state)

    def toggle_columns_visibility_state(self,view,columns):
        """Toggle the columns visibility and store the new state 
           @param view: name of the grid view to be stored 
           @type view: string
           @param columns: name of the columns to be hidden or shown
           @type column: bar separated string
        """
        state = self.load_state()
        hidden_columns = state.get('hidden_columns',{})    
        if not columns:
            hidden_columns[view] = []
            hidden_columns_list = []
        else:
            hidden_columns_list = hidden_columns.get(view,[])    
            columns = columns.split('|')
            for column in columns:

                if column in hidden_columns_list: 
                    hidden_columns_list = [x for x in hidden_columns_list if x !=column]
                else:
                    hidden_columns_list.append(column)

        hidden_columns[view] = hidden_columns_list
        state['hidden_columns'] = hidden_columns
        self.save_state(state)

    def load_columns_visibility_state(self,view):
        """Return a list of columns names for the requested view that should
           not be shown

           @param view: name of the grid view to be stored 
           @type view: string
        """
        state = self.load_state()
        hidden_columns = state.get('hidden_columns',{})    
        return hidden_columns.get(view,[])    

    def load_label_column_for_object(self,objectName):
        """Return the column name to be used as foreignKey label for the
           requested object

           @param objectName: name of the object 
           @type objectName: string
        """
        state = self.load_state()
        lables = state.get('columns',{})    
        return lables.get(objectName,'id')

    def column_label_for_object(self,objectName,columnName):
        """Store the column name to be used as foreignKey label for the 
           requested object

           @param objectName: name of the object 
           @type objectName: string
           @param columnName: name of the column to use as foreignKey label
           @type columnName: string
        """
        state = self.load_state()
        cl= state.get('columns',{})    
        cl[objectName] = columnName
        state['columns'] = cl
        self.save_state(state)

    def load_column_order(self,object_name):
        """If the user has rearrenged the columns order for an object,
           this will return the prefered order as list
           
           @param object_name: name of the object 
           @type object_name: string
        """
        state = self.load_state()
        cols = state.get('order',{})    
        return cols.get(object_name,[])

    def save_column_order(self,object_name,columns_bsv):
        """Save the prefered order of the object's columns

           @param object_name: name of the object 
           @type object_name: string
           @param columns_bsv: bar (|) delimited columns names
           @type columns_bsv: string
        """
        state = self.load_state()
        cl= state.get('order',{})    
        cl[object_name] = columns_bsv.split('|')
        state['order'] = cl
        self.save_state(state)

    def order_columns(self,object_name,cols):
        """Return a rearrenged list of columns as configured by the user

           @param object_name: name of the object 
           @type object_name: string
           @param cols: original list of columns following the default table order 
           @type cols: list
        """
        order = self.load_column_order(object_name)
        if not order: return cols
        c={}
        for col in cols:
            c[col['columnName']] = col
            if col['columnName'] not in order:order.append(col['columnName'])
        rearrenged = []
        for columnName in order:
            if columnName not in c.keys():continue
            rearrenged.append(c[columnName])
        return rearrenged

    def save_model_order(self,models):
        """Store the new order of the listed models"""
        state = self.load_state()
        state['model_order'] = models.split('|')
        self.save_state(state)

    def load_models_order(self):
        state = self.load_state()
        return state.get('model_order',[])    

    def order_models(self,models):
        ordered = self.load_models_order()
        if not ordered: return models
        #add any new models to the ordered list
        for model in models:
            if not model in ordered: ordered.append(model)
        reorderedList = []
        #check that the ordered list don't have delete models
        for model in ordered:
            if model in models: reorderedList.append(model)
        return reorderedList

    """====================== EXPOSED METHODS =============================="""
    def add(self,**v):
        """Create a new instance of an object
            
           @param v: dictionary of submited values
        """
        objectName = v['objectName']
        obj = self.load_object(objectName)
        cols = self.object_columns(objectName)
        params = self.extract_parameters(cols,v)
        if not params: return self.instances(objectName)
        try:
            new_object = obj(**params)
        except Exception, e:
            cherrypy.response.status = 500
            return dict(error=str(e))
        if not new_object: return self.instances(objectName)

        returnlist = self.object_instance(objectName,'%s'% new_object.id)
        returnlist["msg"] = "A new instance of %s was created" % objectName
        return returnlist
    add = turbogears.expose(format="json")(add)

    def update(self,**values):
        """Update the objects properties
           
           @param values: dictionary of key and values, as a bare minimum
                     the name of the object (objectName) and the id
        """
        object_name = values.get('objectName','')
        id = values.get('id','')
        try:
            self.update_object(object_name,id,values)
        except Exception, e:
            cherrypy.response.status = 500
            return dict(error=str(e))
        returnlist = self.object_instances(object_name)
        returnlist["msg"] = "The object was successfully updated"
        return returnlist
    update = turbogears.expose(format="json")(update)

    def remove(self,objectName,id):
        """Remove and instance by id 
           This doesn't handle references (cascade delete)
           
           @param objectName: name of the object 
           @type objectName: string
           @param id: id of the instance to be removed
           @type id: string
        """
        try:
            self.remove_object(objectName,id)
        except Exception, e:
            cherrypy.response.status = 500
            return dict(error=str(e))
        returnlist = self.object_instances(objectName)
        returnlist["msg"] = "The object was successfully deleted"
        return returnlist
    remove = turbogears.expose(format="json")(remove)

    def remove_single_join(self,object_name,id,join_object_name,join_object_id):
        """Remove a single join instance by id 
           This doesn't handle references (cascade delete)
           
           @param object_name: name of the host object 
           @type object_name: string
           @param id: id of the host instance 
           @type id: string
           @param join_object_name: name of the join object 
           @type join_object_name: string
           @param join_object_id: id of the join instance to be removed
           @type join_object_id: string
        """
        self.remove_object(join_object_name,join_object_id)
        return self.object_instance(object_name,id)
    remove_single_join = turbogears.expose(format="json")(remove_single_join)

    def saveModelOrder(self,models): 
        """Save the prefered order of the listed models"""
        self.save_model_order(models)
        return '' #return dummy string, else json will barf
    saveModelOrder = turbogears.expose(format="json")(saveModelOrder)

    def columnOrder(self,objectName,cols):
        """Save the prefered order of the object's columns

           @param objectName: name of the object 
           @type objectName: string
           @param cols: columns names separated by '|' 
           @type cols: string
        """
        self.save_column_order(objectName,cols)
        """return dummy string, else json will barf"""
        return '' 
    columnOrder = turbogears.expose(format="json")(columnOrder)
    
    def instances(self,objectName,start=0):
        """Return a JSON structure containing all instances of the requested object
           
           @param objectName: name of the object 
           @type objectName: string
        """
        return self.object_instances(objectName,start)
    instances = turbogears.expose(format="json")(instances)

    def manageRelatedJoins(self,objectName,id,join,relatedObjectName,**vargs):
        """Return a JSON structure whith a list of related joins for 
           the requested object, and a list of all joins

           @param objectName: name of the object 
           @type objectName: string
           @param id: id of the instance
           @type id: string
           @param join: name of the join (joinMethodName in SQLObject parlance)
           @type join: string
           @param relatedObjectName: otherClassName (in SQLObject parlance)
           @type relatedObjectName: string
        """
        joins = self.object_joins(objectName,id,join,'SORelatedJoin',relatedObjectName)

        joins['allJoins'] = self.object_instances(relatedObjectName)
        return joins
    manageRelatedJoins = turbogears.expose(format="json")(manageRelatedJoins)
   
    def updateJoins(self,objectName,id,join,joinType,joinObjectName,joins):
        """Drop all related joins first, then loop trought the submited joins
           and set the relation
           
           @param objectName: name of the object 
           @type objectName: string
           @param id: id of the instance to be removed
           @type id: string
           @param join: name of the join field (joinMethodName)
           @type join: string
           @param joinType: type of the join (Multiple or Related)
           @type joinType: string
           @param joinObjectName: name of the joined object (otherClassName)
           @type joinObjectName: string
           @param joins: coma delimited string of join instances id's
           @type joins: string
        """
        try:
            obj = self.load_object(objectName)
            inst = obj.get(id)
            
            # get the add/remove method names
            j = [j for j in obj.sqlmeta.joins if (j.joinMethodName == join)][0]
            remove_method = getattr(inst, 'remove' +  j.addRemoveName)
 	    add_method = getattr(inst, 'add' + j.addRemoveName)
 	    
            # remove all joined instances
            for joined_instance in list(getattr(inst, join)):
                remove_method(joined_instance)

            # add the new joined instances
            join_object = self.load_object(joinObjectName)
            joins = joins.split(',')
            for i in joins:
                try: i=int(i) 
                except: continue
                instance = join_object.get(i)
                add_method(instance)
        except Exception, e:
            cherrypy.response.status = 500
            return dict(error=str(e))
        returnlist = self.object_instance(objectName,id); 
        returnlist["msg"] = "The object was successfully updated"
        return returnlist

    updateJoins = turbogears.expose(format="json")(updateJoins)

    def updateColumns(self,objectName,column):
        """Toggle (and store) the state of the requested column in grid view display. 
           
           @param objectName: name of the object 
           @type objectName: string
           @param column: name of the column to be hidden
           @type column: string
        """
        self.toggle_columns_visibility_state(objectName,column)
        return self.object_instances(objectName)
    updateColumns = turbogears.expose(format="json")(updateColumns)

    def updateColumnsJoinView(self,objectName,id,join,joinType,joinObjectName,column):
        """Toggle (and store) the state of the requested column 
           in grid view display for a join view. 
           
           @param objectName: name of the object 
           @type objectName: string
           @param id: id of the 'parent' instance 
           @type id: string
           @param join: name of the join (joinMethodName in SQLObject parlance)
           @type join: string
           @param joinType: name of join type
           @type joinType: string
           @param joinObjectName: otherClassName (in SQLObject parlance)
           @type joinObjectName: string
           @param column: name of the column to be hidden or shown
           @type column: string
        """
        self.toggle_columns_visibility_state('%s_%s'% (objectName,joinObjectName),column)
        return self.object_joins(objectName,id,join,joinType,joinObjectName)
    updateColumnsJoinView = turbogears.expose(format="json")(updateColumnsJoinView)

    def joins(self,objectName,id,join,joinType,joinObjectName):
        """Return a JSON structure containing a list joins for 
           the requested object's joinMethodName

           @param objectName: name of the object 
           @type objectName: string
           @param id: id of the instance
           @type id: string
           @param join: name of the join (joinMethodName in SQLObject parlance)
           @type join: string
           @param joinType: name of join type
           @type joinType: string
           @param joinObjectName: otherClassName (in SQLObject parlance)
           @type joinObjectName: string
        """
        return self.object_joins(objectName,id,join,joinType,joinObjectName)
    joins = turbogears.expose(format="json")(joins)
    
    def object_instance(self,object_name,id):
        obj = self.load_object(object_name)
        inst = obj.get(id)
        cols = self.object_columns(object_name)
        values = []
        for col in cols:
            options = []
            colForID=col.copy()
            colForID['type']=''
            col['object_id']=id
            if col['type'] in ('SOMultipleJoin', 'SOSQLMultipleJoin',
              'SORelatedJoin', 'SOSQLRelatedJoin'):
                col['id'] = id
                try:
                    value = '%s'% len(list(getattr(inst,col['columnName'])))
                except:
                    value = '0'
                col['value'] = {'value':value, 'column':col['columnName']}
            elif col['type'] in ('SOForeignKey', 'SOSingleJoin'):
                try:
                    otherClass = getattr(inst,col['columnName'])
                    col['id'] = self.object_field(inst,colForID)
                    col['id']['value'] = '%s'% otherClass.id
                    try:
                        label_value = '%s'% getattr(otherClass,col['labelColumn'])
                    except AttributeError:
                        label_value = self.object_representation(otherClass)
                    label_value = self.encode_label(label_value)
                    col['value'] = {'column':col['columnName'],'value':label_value }
                except AttributeError:
                    col['id']='__default_none__'
                    col['value']={'column':col['columnName'],
                                  'value':'None',
                                  'id':'__default_none__'}
            else:
                col['id'] = self.object_field(inst,colForID)
                col['value'] = self.object_field(inst,col)
            col['objectName'] = object_name
            values.append(col) 
        return dict(objectName=object_name,id=id,values=values)

    def instance(self,objectName,id):
        """Return a JSON structure containing the columns and field values for
           the requested object

           @param objectName: name of the object 
           @type objectName: string
           @param id: id of the instance
           @type id: string
        """
        return self.object_instance(objectName,id)
    instance = turbogears.expose(format="json")(instance)

    def columnsForLabel(self,objectName,foreignObjectName,foreignKeyName):
        """Return a JSON structure with a list of columns to use as foreignKey label

           @param objectName: name of the object 
           @type objectName: string
           @param foreignObjectName: name of the object the foreignKey refers to
           @type foreignObjectName: string
           @param foreignKeyName: name of the object foreignKey field
           @type foreignKeyName: string
        """
        cols = [{'columnName':col['columnName'],'title':col['title']}    
                        for col in self.object_columns(foreignObjectName) 
                        if col['type'] !='SOMultipleJoin' 
                     ]
        return dict(columns=cols,
                    foreignKeyName=foreignKeyName,
                    foreignKeyColumnForLabel= self.load_label_column_for_object(foreignObjectName), 
                    foreignObjectName=foreignObjectName,
                    objectName=objectName)
    columnsForLabel = turbogears.expose(format="json")(columnsForLabel)

    def setColumnForLabel(self,objectName,foreignObjectName,foreignKeyName,columnName):
        """Exposed method that let you store the column name to be used as 
           foreignKey label for the requested object

           @param objectName: name of the object 
           @type objectName: string
           @param foreignObjectName: name of the object the foreignKey refers to
           @type foreignObjectName: string
           @param foreignKeyName: name of the object foreignKey field
           @type foreignKeyName: string
           @param columnName: name of the column to use as foreignKey label
           @type columnName: string
        """
        self.column_label_for_object(foreignObjectName,columnName)
        return self.columns(objectName)
    setColumnForLabel = turbogears.expose(format="json")(setColumnForLabel)

    def columns(self,objectName,**kv):
        """Return JSON structure containing a list of column properties

           @param objectName: name of the object 
           @type objectName: string
        """
        reorder = not kv.has_key('foreignObjectName')
        return dict(objectName=objectName,
                                columns=self.object_columns(objectName),
                                reorder=reorder,
                                reordering=kv )
    columns = turbogears.expose(format="json")(columns)

    def list(self):
        """Return JSON structure containing a list of available objects"""
        objs = []
        for m in dir(self.model):
            if m in ('SQLObject','InheritableSQLObject'): continue
            c = getattr(self.model,m)
            if isinstance(c, type) and issubclass(c,sqlobject.SQLObject):
                objs.append(m)
        objs = self.order_models(objs)
        return dict(SQLObjects=objs)
    list = turbogears.expose(format="json")(list)

    def models(self):
        objs = []
        for m in dir(self.model):
            if m in ('SQLObject','InheritableSQLObject'): continue
            c = getattr(self.model,m)
            if isinstance(c, type) and issubclass(c,sqlobject.SQLObject): objs.append(m)
        return self.order_models(objs)

    def register_static_directory(self):
        static_directory = pkg_resources.resource_filename(__name__,'static')
        turbogears.config.update( 
                                { '/tg_toolbox/catwalk': {
                                                            'static_filter.on':True,
                                                            'static_filter.dir':static_directory
                                                          }
                                }
                               )

    def index(self):
        """Main CatWalk page, import the proper client side libraries and set up the placeholder
           for the dynamic elements
        """
        return dict(models=self.models())
    index = turbogears.expose(template='turbogears.toolbox.catwalk.catwalk')(index)



import os
import itertools
import pkg_resources
import warnings
from turbogears import view, validators, startup, config
from turbogears.util import setlike, to_unicode, copy_if_mutable
from turbogears.widgets.meta import MetaWidget, load_kid_template

try: 
    set
except NameError: 
    from sets import Set as set

__all__ = ["load_widgets", "all_widgets", "Widget", "CompoundWidget",
           "WidgetsList", "register_static_directory",
           "static", "Resource", "Link", "CSSLink", "JSLink", 
           "Source", "CSSSource", "JSSource", "js_location", "mochikit",
           "WidgetDescription", "set_with_self"]


class Enum(set):
    """Enum used at js_locations which is less strict than 
    ``turbogears.utils.Enum`` and serves our purposes as well as allowing
    any object with ``retrieve_javascript``, ``retrieve_css``, and 
    ``location`` attributes to provide resources to the template when
    scanned in ``turbogears.controllers._process_output``.

    
    Example:

        >>> locations = Enum('bodytop', 'bodybottom', head')
        >>> locations.head == locations.head
        True
        >>> locations.head == locations.bodybottom
        False
        >>> locations.head in locations
        True
        >>> locations.foo
        Traceback (most recent call last):
        ...
        AttributeError

    """
    def __init__(self, *args):
        set.__init__(self, args)

    def __getattr__(self, name):
        if name in self:
            return name
        raise AttributeError

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, ', '.join(map(repr, self)))

# Keeps a count of the each declared widget in a WidgetsList instance
# so their order is preserved.
counter = itertools.count()

# Load all widgets provided by the widget entry point
def load_widgets():
    for widget_mod in pkg_resources.iter_entry_points("turbogears.widgets"):
        widget_mod.load()

all_widgets = set()

PlainHTML = load_kid_template("""
                              <html xmlns:py="http://purl.org/kid/ns#"
                                  py:replace="elements" 
                              />
                              """,
                              modname="turbogears.widgets.plainhtml"
                             )[0]

#############################################################################
# Widgets base classes                                                      #
#############################################################################
class Widget(object):
    """
    A TurboGears Widget.
    
   '__init__' and 'update_params' are the only methods you might need to
   care extending.

    Attributes you should know about:
    
    name      : The name of the widget.
    template  : Kid template for the widget.
    default   : Default value for the widget.
    css       : List of CSSLinks and CSSSources for the widget. These
                will be automatically pulled and inserted in the
                page template that displays it.
    javascript: List of JSLinks and JSSources for the widget. Same as css.
    is_named  : A property returning True if the widget has overrided it's
                default name.
    params    : All parameter names listed here will be treated as special
                parameters. This list is updated by the metaclass and 
                always contains *all* params from the widget itself and
                all it's bases. Can be used as a quick reminder of all
                the params your widget has at it's disposal. They all
                behave the same and have the same priorities regarding their
                overridal. Read on...
    params_doc: A dictionary containing 'params' names as keys and their
                docstring as value. For documentation at the widget browser.

   All initialization parameters listed at the class attribute "params" can be 
   defined as class attributes, overriden at __init__ or at display time. They 
   will be treated as special params for the widget, which means:
   
   1) You can fix default values for them when sublcassing Widget.

   2) If passed as **params to the constructor, the will be bound automatically
      to the widget instance, taking preference over any class attributes 
      previously defined. Mutable attributes (dicts and lists) defined as class
      attributes are safe to modify as care is taken to copy them so the class 
      attribute remains unchanged. 

   3) They can be further overrided by passing them as keyword args to 
      display(). This will only affect that display() call in a thread-safe 
      way.

   4) A callable can be passed and it will be called automatically when sending
      variables to the template. This can be handy to pick up parameters which 
      change in every request or affect many widgets simultaneously.
    """

    __metaclass__ = MetaWidget

    name = "widget"
    template = None
    default = None
    css = []
    javascript = []
    params = []
    params_doc = {}

    def __init__(self, name=None, template=None, default=None, **params):
        """
        All initialization has to take place in this method. It's not 
        thread-safe to mutate widget's attributes outside this method or 
        anytime after widget's first display.

        *Must* call super(MyWidget, self).__init__(*args, **kw) cooperatively,
        unless, of course, your know what you're doing. Preferably this should
        be done before any actual work is done in the method.

        Parameters:

        name     : The widget's name. In input widgets, this will also be the
                   name of the variable that the form will send to the 
                   controller. This is the only param that is safe to pass as a 
                   positional argument to __init__.
        template : The template that the widget should use to display itself.
                   Currently only Kid templates are supported. You can both
                   initialize with a template string or with the path to a
                   file-base template: "myapp.templates.widget_tmpl"
        default  : Default value to display when no value is passed at display
                   time.
        **params : Keyword arguments specific to your widget or to any of it's
                   bases. If listed at class attribute 'params' the will be
                   bound automatically to the widget instance.

        Note: Do not confuse these parameters with parameters listed at 
        "params". Some widgets accept parameters at the constructor which are
        not listed params, these parameter won't be passed to the template, be
        automatically called, etc..
        """
        self._declaration_counter = counter.next()
        if name:
            self.name = name
        if template:
            (self.template_c, self.template) = load_kid_template(template)
        if default:
            self.default = default
        # logic for managing the params attribute
        for param in self.__class__.params:
            if param in params:
                # make sure we don't keep references to mutables
                setattr(self, param, copy_if_mutable(params.pop(param)))
            else:
                if hasattr(self, param):
                    # make sure we don't alter mutable class attributes
                    value = copy_if_mutable(getattr(self.__class__, param),
                                            True)
                    if value[1]:
                        # re-set it only if mutable
                        setattr(self, param, value[0])
                else:
                    setattr(self, param, None)
        for unused in params.iterkeys():
            warnings.warn('keyword argument "%s" is unused at %r instance' % (
                unused, self.__class__.__name__))

    def adjust_value(self, value, **params):
        """
        Adjusts the value sent to the template on display.
        """
        if value is None:
            return self.default
        else:
            return value

    def update_params(self, params):
        """
        This method will have the last chance to update the variables sent to 
        the template for the specific request. All parameters listed at class 
        attribute 'params' will be available at the 'params' dict this method 
        receives.

        *Must* call super(MyWidget, self).update_params(params) cooperatively,
        unless, of course, your know what you're doing. Preferably this should
        be done before any actual work is done in the method.
        """
        pass

    # The following methods are needed to be a well behaved widget, however,
    # there is rarely the need to override or extend them if inheritting
    # directly or indirectly from Widget.

    # update_data has been deprecated
    update_data = update_params

    def __call__(self, *args, **params):
        """
        Delegate to display. Used as an alias to avoid tiresome typing
        """
        return self.display(*args, **params)

    def display(self, value=None, **params):
        """
        Display the widget in a Kid template. Returns an elementtree node 
        instance. If you need serialized output in a string call 'render'
        instead.
        Probably you will not need to override or extend if inhertitting from
        Widget.

        @params:

        value    : The value to display in the widget.
        **params : Extra parameters specific to the widget. All keyword params
                   supplied will pass through the update_params method which will 
                   have a last chance to modify them before reaching the
                   template. 
        """

        if not getattr(self, 'template_c', False):
            warnings.warn("Widget instance '%r' has no template defined" % self)
            return None
        # logic for managing the params attribute
        for param in self.__class__.params:
            if param in params:
                param_value = params[param]
                if callable(param_value):
                    param_value = param_value()
            else:
                # if the param hasn't been overridden (passed as a keyword
                # argument inside **params) put the corresponding instance
                # value inside params.
                param_value = getattr(self, param, None)
            # make sure we don't pass a reference to mutables
            params[param] = copy_if_mutable(param_value)
        params["name"] = self.name
        params["value"] = to_unicode(self.adjust_value(value, **params))
        self.update_params(params)
        # update_data has been deprecated
        self.update_data(params)
        return view.engines.get('kid').transform(params, self.template_c)

    def render(self, value=None, format="html", **params):
        """
        Exactly the same as display() but return serialized output instead.
        Useful for debugging or to display the widget in a non-Kid template like
        Cheetah, STAN, ...
        """ 
        elem = self.display(value, **params)
        t = PlainHTML(elements=elem)
        return t.serialize(output=format, fragment=True)

    def retrieve_javascript(self):
        """
        Return a setlike instance with all the JSLinks and JSSources the
        widget needs.
        """ 
        scripts = setlike()
        for script in self.javascript:
            scripts.add(script)
        return scripts

    def retrieve_css(self):
        """
        Return a setlike instance with all the CSSLinks and CSSSources the
        widget needs.
        """ 
        css = setlike()
        for cssitem in self.css:
            css.add(cssitem)
        return css

    def _get_is_named(self):
        """
        Return True if the widget has overrided it's default name, else False.
        """
        if self.name != "widget":
            return True
        else:
            return False
    is_named = property(_get_is_named)

    def __setattr__(self, key, value):
        if self._locked:
            raise ValueError, \
                "It is not threadsafe to modify widgets in a request"
        else:
            return super(Widget, self).__setattr__(key, value)

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, ', '.join(
            ["%s=%r" % (var, getattr(self, var))
                for var in ['name'] + self.__class__.params]))

class CompoundWidget(Widget):
    """A widget that can contain other widgets.

    A compund widget is a widget that can group several widgets to make a 
    complex widget. Child widget names must be listed at ther widget's
    ``member_widgets`` attribute.
    """
    compound = True
    member_widgets = []

    def __init__(self, *args, **kw):
        # logic for managing the member_widgets attribute
        for member in self.__class__.member_widgets:
            if member in kw:
                setattr(self, member, kw.pop(member))
            elif not hasattr(self, member):
                setattr(self, member, None)
        super(CompoundWidget, self).__init__(*args, **kw)

    def iter_member_widgets(self):
        """Iteratetes over all the widget's children"""
        for member in self.__class__.member_widgets:
            attr = getattr(self, member, None)
            if isinstance(attr, list):
                for widget in attr:
                    yield widget
            elif attr is not None:
                yield attr 

    def display(self, value=None, **params):
        params["member_widgets_params"] = params.copy()
        # logic for managing the member_widgets attribute
        for member in self.__class__.member_widgets:
            params[member] = getattr(self, member, None)
        return super(CompoundWidget, self).display(value, **params)

    def update_params(self, d):
        super(CompoundWidget, self).update_params(d)
        d['value_for'] = lambda f: self.value_for(f, d['value'])
        widgets_params = d['member_widgets_params']
        d['params_for'] = lambda f: self.params_for(f, **widgets_params)
 
    def value_for(self, item, value):
        """
        Pick up the value for a given member_widget 'item' from the 
        value dict passed to this widget.
        """
        name = getattr(item, "name", item)
        if isinstance(value, dict):
            return value.get(name)
        else:
            return None
 
    def params_for(self, item, **params):
        """
        Pick up the params for the given member_widget 'item' from
        the params dict passed to this widget.
        """
        name = getattr(item, "name", item)
        item_params = {}
        for k,v in params.iteritems():
            if isinstance(v, dict):
                if name in v:
                    item_params[k] = v[name]
        return item_params

    def retrieve_javascript(self):
        """
        Retrieve the javascript for all the member widgets and
        get an ordered union of them.
        """
        scripts = setlike()
        for script in self.javascript:
            scripts.add(script)
        for widget in self.iter_member_widgets():
            for script in widget.retrieve_javascript():
                scripts.add(script)
        return scripts

    def retrieve_css(self):
        """
        Retrieve the css for all the member widgets and
        get an ordered union of them.
        """
        css = setlike()
        for cssitem in self.css:
            css.add(cssitem)
        for widget in self.iter_member_widgets():
            for cssitem in widget.retrieve_css():
                css.add(cssitem)
        return css


#############################################################################
# Declarative widgets support                                               #
#############################################################################
class MetaWidgetsList(type):
    """
    Metaclass for WidgetLists. Takes care that the resulting WidgetList
    has all widgets in the same order as they were declared.
    """
    def __new__(meta, class_name, bases, class_dict):
        declared_widgets = []
        for name, value in class_dict.items():
            if isinstance(value, Widget):
                if not value.is_named:
                    value.name = name
                declared_widgets.append(value)
                del class_dict[name]
        declared_widgets.sort(lambda a, b: cmp(a._declaration_counter,
                                               b._declaration_counter))
        cls = type.__new__(meta, class_name, bases, class_dict)
        cls.declared_widgets = declared_widgets
        return cls

class WidgetsList(list):
    """
    A widget list. That's really all. A plain old list that you can
    declare as a classs with widgets ordered as attributes. Syntactic
    sugar for an unsweet world.
    """
    __metaclass__ = MetaWidgetsList

    def __init__(self):
        super(WidgetsList, self).__init__()
        self.extend(self.declared_widgets)
        if len(self) == 0:
            warnings.warn("You have declared an empty WidgetsList")

#############################################################################
# CSS, JS and mochikit stuff                                                #
#############################################################################
def register_static_directory(modulename, directory):
    """
    Sets up a static directory for JavaScript and css files. You can refer
    to this static directory in templates as ${tg.widgets}/modulename
    """
    directory = os.path.abspath(directory)
    config.update({'/tg_widgets/%s' % modulename :
                   {
                       'static_filter.on' : True,
                       'static_filter.dir' : directory
                   }
                  }
                 )

static = "turbogears.widgets"

register_static_directory(static,
                          pkg_resources.resource_filename(__name__,
                                                          "static")
                         )

register_static_directory("turbogears",
                          pkg_resources.resource_filename("turbogears",
                                                          "static")
                         )

def set_with_self(self):
    theset = setlike()
    theset.add(self)
    return theset

class Resource(Widget):
    """
    A resource for your widget, like a link to external JS/CSS or inline
    source to include at the template the widget is displayed.
    """
    pass

class Link(Resource):
    def __init__(self, mod, *args, **kw):
        super(Link, self).__init__(*args, **kw)
        self.mod = mod

    def update_params(self, d):
        super(Link, self).update_params(d)
        d["link"] = "/%stg_widgets/%s/%s" % (startup.webpath,
                                             self.mod,
                                             self.name)

    def __hash__(self):
        return hash(self.mod + self.name)

    def __eq__(self, other):
        return self.mod == getattr(other, "mod", None) and \
            self.name == getattr(other, "name", None)


class CSSLink(Link):
    template = """
    <link rel="stylesheet" 
        type="text/css" 
        href="$link" 
        media="$media"
    />
    """
    params = ["media"]
    params_doc = {'media': 'Specify the media attribute for the css link tag'}
    media = "all"

    retrieve_css = set_with_self

js_location = Enum('head', 'bodytop', 'bodybottom')

class JSLink(Link):
    template = """
    <script type="text/javascript" src="$link"></script>
    """

    location = js_location.head

    def __init__(self, *args, **kw):
        location = kw.pop('location', None)
        super(JSLink, self).__init__(*args, **kw)
        if location:
            if location not in js_location:
                raise ValueError, "JSLink location should be in %s" % js_location
            self.location = location
        
    retrieve_javascript = set_with_self

mochikit = JSLink("turbogears", "js/MochiKit.js")

class Source(Resource):
    params = ["src"]

    def __init__(self, src, *args, **kw):
        super(Source, self).__init__(*args, **kw)
        self.src = src

    def __hash__(self):
        return hash(self.src)

    def __eq__(self, other):
        return self.src == getattr(other, "src", None)

class CSSSource(Source):
    template = """
    <style type="text/css" media="$media">$src</style>
    """
    params = ["media"]
    params_doc = {'src': 'The CSS source for the link',
                  'media' : 'Specify the media the css source link is for'}
    media = "all"

    retrieve_css = set_with_self

class JSSource(Source):
    template = """
    <script type="text/javascript">$src</script>
    """
    location = js_location.head

    def __init__(self, src, location=None):
        if location:
            if location not in js_location:
                raise ValueError, "JSSource location should be in %s" % js_location
            self.location = location
        super(JSSource, self).__init__(src)

    retrieve_javascript = set_with_self

#############################################################################
# Classes for supporting the toolbox widget browser                         #
#############################################################################
just_the_widget = load_kid_template("""
                                    <div xmlns:py="http://purl.org/kid/ns#"
                                      py:content="for_widget.display()"
                                    />
                                    """
                                   )[0]

class MetaDescription(MetaWidget):
    """
    Metaclass for widget descriptions. Makes sure the widget browser knows
    about all of them as soon as they come into existence.
    """
    def __init__(cls, name, bases, dct):
        super(MetaDescription, cls).__init__(name, bases, dct)
        register = dct.get("register", True)
        if name != "WidgetDescription" and register:
            all_widgets.add(cls)

class WidgetDescription(CompoundWidget):
    """
    A description for a Widget. Make's the 'for_widget' widget appear in the 
    browser.
    It's a nice way to show off to your friends your coolest new widgets and
    to have a testing platform while developing them.
    """
    __metaclass__ = MetaDescription

    template = just_the_widget
    for_widget = None
    member_widgets = ["for_widget"]
    show_separately = False

    def _get_name(self):
        return self.for_widget_class.__name__
    name = property(_get_name)

    def _get_widget_class(self):
        return self.for_widget.__class__
    for_widget_class = property(_get_widget_class)

    def _get_description(self):
        return self.for_widget_class.__doc__
    description = property(_get_description)

    def _get_full_class_name(self):
        cls = self.for_widget_class
        return "%s.%s" % (cls.__module__, cls.__name__)
    full_class_name = property(_get_full_class_name)

    def _get_source(self):
        import inspect
        return inspect.getsource(self.__class__)
    source = property(_get_source)

    def retrieve_css(self):
        return self.for_widget.retrieve_css()

    def retrieve_javascript(self):
        return self.for_widget.retrieve_javascript()

class CoreWD(WidgetDescription):
    register = False

    def _get_full_class_name(self):
        cls = self.for_widget_class
        return "turbogears.widgets.%s" % (cls.__name__)
    full_class_name = property(_get_full_class_name)

class RenderOnlyWD(WidgetDescription):
    register = False
    template = """
    <div>
        This widget will render like that:<br/><br/>
        <tt class="rendered">${for_widget.render(value)}</tt>
    </div>
    """
    
    def retrieve_javascript(self):
        return setlike()
    
    def retrieve_css(self):
        return setlike()

#############################################################################
# CSS and JS WidgetDescription's                                            #
#############################################################################
class CSSLinkDesc(CoreWD, RenderOnlyWD):
    name = "CSS Link"
    for_widget = CSSLink("turbogears", "css/yourstyle.css")

class JSLinkDesc(CoreWD, RenderOnlyWD):
    name = "JS Link"
    for_widget = JSLink("turbogears", "js/yourscript.js")

class CSSSourceDesc(CoreWD, RenderOnlyWD):
    name = "CSS Source"
    for_widget = CSSSource("""body { font-size:12px; }""")

class JSSourceDesc(CoreWD, RenderOnlyWD):
    name = "JS Source"
    for_widget = JSSource("document.title = 'Hello World';")



import cherrypy
import turbogears

def set_identity_errors( errors ):
    if isinstance( errors, basestring ):
        errors= [errors];
    cherrypy.request.identity_errors= list(errors)

def get_identity_errors():
    return getattr( cherrypy.request, "identity_errors", [] )


class IdentityException(Exception):
    '''
    Base class for all Identity exceptions.
    '''
    pass

class RequestRequiredException(IdentityException):
    '''
    An attempt was made to use a facility of Identity that requires the
    presence of an HTTP request.
    '''

    def __str__(self):
        return "An attempt was made to use a facility of the TurboGears " \
               "Identity Management framework that relies on an HTTP request "\
               "outside of a request."

        
class IdentityManagementNotEnabledException(IdentityException):
    """ User forgot to enable Identity management """
    
    def __str__(self):
        return "An attempt was made to use a facility of the TurboGears " \
               "Identity Management framework but identity management hasn't " \
               "been enabled in the config file [via identity.on]."
    
    
class IdentityConfigurationException(IdentityException):
    '''
    Exception thrown when the Identity management system hasn't been configured
    correctly. Mostly, when failure_url is not specified.
    '''
    args = ()
    def __init__(self, message):
        self.message= message
        
    def __str__(self):
        return self.message

class IdentityFailure(cherrypy.InternalRedirect, IdentityException):
    '''
    Exception thrown when an access control check fails.
    '''
    def __init__(self, errors):
        '''
        Set up the identity errors on the request and get the URL from the config
        '''
        set_identity_errors( errors )
        url= turbogears.config.get( "identity.failure_url", None )
        if url is None:
            msg= "Missing URL for identity failure"
            raise IdentityConfigurationException( msg )
        if callable(url):
            url= url( errors )
        # super(IdentityFailure,self).__init__(url) seems to not work...?
        cherrypy.InternalRedirect.__init__(self, url)



#!/usr/bin/env python
names = ["aces", "astonishing", "astounding", 
    "best", "breathtaking", "cool", 
    "doozie", "extravagant", "fab", "fantastic", 
    "groovy", 
    "immense", "in spades", "inconceivable",
    "legendary", "marvelous", "mind-blowing", "out-of-this-world", 
    "outrageous", "phenomenal", "primo", "prodigious", 
    "remarkable", "spectacular", "striking", 
    "super", "superb", "terrific", "top drawer", "tops", "turn-on", 
    "unbelievable", "wicked", "wondrous"]

import random
print random.choice(names)


import cherrypy
import sha
import datetime
import time
import base64

try:
    from sqlobject import *
except ImportError:
    pass

import turbogears
from turbogears.identity import create_default_provider
from turbogears.identity import set_current_identity
from turbogears.identity import set_current_provider
from turbogears.identity import set_login_attempted

from turbogears.identity.exceptions import *

from turbogears import visit

import logging
log = logging.getLogger("turbogears.identity")

# Interface for the TurboGears extension
def start_extension():
    # Bail out if the application hasn't enabled this extension
    if not turbogears.config.get( "identity.on", False ):
        return
    # Identity requires that Visit tracking be enabled
    if not turbogears.config.get( "visit.on", False ):
        raise IdentityConfigurationException( "Visit tracking must be enabled (visit.on)" )
        
    log.info( "Identity starting" )
    # Temporary until tg-admin can call create_extension_model
    create_extension_model()
    # Register the plugin for the Visit Tracking framework
    visit.enable_visit_plugin( IdentityVisitPlugin() )

    
def shutdown_extension():
    # Bail out if the application hasn't enabled this extension
    if not turbogears.config.get( "identity.on", False ):
        return
    log.info( "Identity shutting down" )
    pass


def create_extension_model():
    provider= create_default_provider()
    provider.create_provider_model()
    

class IdentityVisitPlugin(object):
    def __init__(self):
        log.info( "Identity visit plugin initialised" )
        get=turbogears.config.get

        self.provider= create_default_provider()
                    
        # When retrieving identity information from the form, use the following
        # form field names. These fields will be removed from the post data to
        # prevent the controller from receiving unexpected fields.
        self.user_name_field= get( "identity.form.user_name", "user_name" )
        self.password_field= get( "identity.form.password", "password" )
        self.submit_button_name= get( "identity.form.submit", "login" )
        
        # Sources for identity information and the order in which they should be
        # checked. These terms are mapped to methods by prepending
        # "identity_from_".
        sources= get( "identity.source", "form,http_auth,visit" ).split(",")
        self.identity_sources= []
        for s in sources:
            try:
                source_method= getattr( self, "identity_from_" + s )
            except AttributeError:
                raise IdentityConfigurationException( "Invalid identity source: %s" % s )
            self.identity_sources.append( source_method )

    def identity_from_request(self, visit_key):
        '''
        Retrieve identity information from the HTTP request. Checks first for
        form fields defining the identity then for a cookie. If no identity
        is found, returns an anonymous identity.
        '''
        identity= None
        log.debug( "Retrieving identity for visit: %s", visit_key )
        for source in self.identity_sources:
            identity= source(visit_key)
            if identity:
                return identity

        log.debug( "No identity found" )
        # No source reported an identity
        identity= self.provider.anonymous_identity()
        return identity
    
    def decode_basic_credentials( self, credentials ):
        '''
        Decode base64 user_name:password credentials used in Basic Auth. Returned
        with username in element 0 and password in element 1.
        '''
        return base64.decodestring( credentials.strip() ).split( ":" )
        
    def identity_from_http_auth( self, visit_key ):
        '''
        Only basic auth is handled at the moment.
        '''
        try:
            authorisation= cherrypy.request.headers['Authorization']
        except KeyError:
            return None
        (authScheme,schemeData)= authorisation.split( " ", 1 )
        # Only basic is handled at the moment
        if "basic" != authScheme.lower():
            log.error( "HTTP Auth is not basic" )
            return None
        # decode credentials
        (user_name,password)=self.decode_basic_credentials( schemeData )
        set_login_attempted( True )
        return self.provider.validate_identity( user_name, password, visit_key )

    def identity_from_visit( self, visit_key ):
        return self.provider.load_identity( visit_key )
    
    def identity_from_form( self, visit_key ):
        '''
        Inspect the form to pull out identity information. Must have fields for
        user name, password, and a login submit button.
        
        Returns an identity dictionary or none if the form contained no identity
        information or the information was incorrect.
        '''
        params= cherrypy.request.params
        # only try to process credentials for login forms
        if params.has_key(self.submit_button_name):
            try:
                # form data contains login credentials
                user_name = params.pop(self.user_name_field)
                pw = params.pop(self.password_field)
                # just lose the submit button to prevent passing to final controller
                submit= params.pop(self.submit_button_name, None)
                submit_x = params.pop("%s.x" % self.submit_button_name, None)
                submit_y = params.pop("%s.y" % self.submit_button_name, None)
                set_login_attempted( True )
                identity= self.provider.validate_identity( user_name, pw, visit_key )
                if identity is None:
                    log.warning( "The credentials specified weren't valid" )
                    return None

                return identity
            except KeyError:
                return None
        else:
            return None
        
    def record_request( self, visit ):
        # default to keeping the identity filter off
        if not turbogears.config.get( "identity.on", True ):
            log.debug( "Identity is not enabled. Setting current identity to None" )
            set_current_identity( None )
            return

        try:
            identity= self.identity_from_request(visit.key)
        except IdentityException, e:
            log.exception( "Caught exception while getting identity from request" )
            errors= [str(e)]
            raise IdentityFailure( errors )
        
        log.debug( "Identity is available..." )
        # stash the user in the thread data for this request
        set_current_identity( identity )
        set_current_provider( self.provider )





import unittest

import cherrypy

from turbogears.controllers import error_handler, exception_handler, \
                                   expose, validate, RootController, Controller
from turbogears.errorhandling import FailsafeSchema
from turbogears.util import bind_args
from turbogears import validators
from turbogears import testutil


def _errors_to_str(errors):
    if isinstance(errors, dict):
        return dict(map(lambda (k,v): (k, str(v)), errors.iteritems()))
    else:
        return str(errors)

class MyRoot(RootController):

    def defaulterrorhandler(self, tg_source, tg_errors, tg_exceptions,
                            *args, **kw):
        return dict(title="Default error handler",
                    errors=_errors_to_str(tg_errors), args=args, kw=kw)

    def specialisederrorhandler(self, tg_source, tg_errors, *args, **kw):
        return dict(title="Specialised error handler",
                    errors=_errors_to_str(tg_errors), args=args, kw=kw)

    def defaulterror(self, bar=""):
        return dict(title="Default error provider")
    defaulterror = error_handler(defaulterrorhandler)(defaulterror)
    defaulterror = validate(validators={"bar":validators.StringBoolean()})(
                            defaulterror)
    defaulterror = expose()(defaulterror)

    def specialisederror(self, bar="", baz=""):
        return dict(title="Specialised error provider")
    specialisederror = error_handler(defaulterrorhandler)(
                                             specialisederror)
    specialisederror = error_handler(specialisederrorhandler,
                                     "'baz' in tg_errors")(specialisederror)
    specialisederror = validate(validators={"bar":validators.Int(not_empty=True),
                                            "baz":validators.Email()})(
                                specialisederror)
    specialisederror = expose()(specialisederror)

    def exceptionerror(self):
        raise Exception("Exception 1")
    exceptionerror = exception_handler(defaulterrorhandler)(exceptionerror)
    exceptionerror = expose()(exceptionerror)

    def exceptionerror2(self):
        raise Exception("Exception 2")
    exceptionerror2 = exception_handler(exceptionerror2)(exceptionerror2)
    exceptionerror2 = expose()(exceptionerror2)

    def recursiveerror(self, tg_errors=None, bar=""):
        if tg_errors:
            return dict(title="Recursive error handler")
        else:
            return dict(title="Recursive error provider")
    recursiveerror = error_handler()(recursiveerror)
    recursiveerror = validate(validators={"bar":validators.Int(not_empty=True)})(
                              recursiveerror)
    recursiveerror = expose()(recursiveerror)

    def impliciterror(self, tg_errors=None, bar=""):
        if tg_errors:
            return dict(title="Implicit error handler",
                        tg_errors=str(tg_errors))
        else:
            return dict(title="Implicit error provider")
    impliciterror = validate(validators={"bar":validators.Int(not_empty=True)})(
                             impliciterror)
    impliciterror = expose()(impliciterror)

    def normalmethod(self):
        return dict(title="Normal method")
    normalmethod = expose()(normalmethod)

    def normalmethodcaller(self, bar=""):
        return dict(title="Normal method caller")
    normalmethodcaller = error_handler(normalmethod)(normalmethodcaller)
    normalmethodcaller = validate(validators={
                                  "bar":validators.StringBoolean()})(
                         normalmethodcaller)
    normalmethodcaller = expose()(normalmethodcaller)

    def infiniteloop(self):
        try:
            self.exceptionerror2()
        except Exception, e:
            return dict(title=str(e))
        else:
            return dict(title="Infinite loop provider")
    infiniteloop = expose()(infiniteloop)

    def positionalargs(self, first, second, *args, **kw):
        self.first = first
        self.second = second
        self.third = args[0]
        return dict(title="Positional arguments", first=first, second=second,
                    args=args, bar=kw["bar"])
    positionalargs = error_handler(defaulterrorhandler)(
                                   positionalargs)
    positionalargs = validate(validators={"bar":validators.StringBoolean(),
                                          "second":validators.Int(not_empty=True)})(
                              positionalargs)
    positionalargs = expose()(positionalargs)

    def missingargs(self, bar=""):
        return dict(title="Missing args provider")
    missingargs = error_handler(defaulterrorhandler)(missingargs)
    missingargs = validate(validators={"bar":validators.Int(not_empty=True)})(missingargs)
    missingargs = expose()(missingargs)

    def nohandler2(self, bar=""):
        return dict(title="No handler inner")
    nohandler2 = validate(validators={"bar":validators.Int(not_empty=True)})(nohandler2)
    nohandler2 = expose()(nohandler2)

    def nohandler(self):
        try:
            self.nohandler2("abc")
        except Exception, NotImplementedError:
            return dict(title="Exception raised")
        else:
            return dict(title="No handler")
    nohandler = expose()(nohandler)

    def simpleerrorhandler(self, baz=None):
         return dict(title="Default error handler", baz=baz)

    def bindargs(self, bar=""):
        return dict(title="Bind arguments to error handler")
    bindargs = error_handler(bind_args(baz=123)(simpleerrorhandler))(bindargs)
    bindargs = validate(validators={"bar":validators.Int(not_empty=True)})(bindargs)
    bindargs = expose()(bindargs)

    def notexposed(self, bar, tg_errors = None):
        if tg_errors:
            return dict(title="Not exposed error", bar=bar)
        else:
            return dict(title="Not exposed", bar=bar)
    notexposed = validate(validators={"bar":validators.Int(not_empty=True)})(notexposed)

    def notexposedcaller(self, foo="", bar="", baz=""):
        return self.notexposed(bar)
    notexposedcaller = expose()(notexposedcaller)

    def continuation(self, tg_source):
        self.continuation = True
        return tg_source(self)

    def continuationcaller(self, bar=""):
        return dict(title="Continuation caller")
    continuationcaller = error_handler(continuation)(continuationcaller)
    continuationcaller = validate(validators={"bar":validators.Int(not_empty=True)})(
                                  continuationcaller)
    continuationcaller = expose()(continuationcaller)

    def nest(self, bar=""):
        return dict(title="Nested")
    nest = error_handler(defaulterrorhandler)(nest)
    nest = validate(validators={"bar":validators.Int(not_empty=True)})(nest)
    nest = expose()(nest)

    def failsafenone(self, tg_errors=None, bar="", baz=""):
        return dict(title="No failsafe", bar=bar, baz=baz)
    failsafenone = validate(validators={"bar":validators.Int(not_empty=True),
        "baz":validators.Int(not_empty=True)})(failsafenone)
    failsafenone = expose()(failsafenone)

    def failsafevaluesdict(self, tg_errors=None, bar="", baz=""):
        return dict(title="Failsafe values-dict", bar=bar, baz=baz)
    failsafevaluesdict = validate(validators={"bar":validators.Int(not_empty=True),
        "baz":validators.Int(not_empty=True)}, failsafe_schema=FailsafeSchema.values,
        failsafe_values={"bar":1, "baz":2})(failsafevaluesdict)
    failsafevaluesdict = expose()(failsafevaluesdict)

    def failsafevaluesatom(self, tg_errors=None, bar="", baz=""):
        return dict(title="Failsafe values-atom", bar=bar, baz=baz)
    failsafevaluesatom = validate(validators={"bar":validators.Int(not_empty=True),
        "baz":validators.Int(not_empty=True)}, failsafe_schema=FailsafeSchema.values,
        failsafe_values=13)(failsafevaluesatom)
    failsafevaluesatom = expose()(failsafevaluesatom)

    def failsafemaperrors(self, tg_errors=None, bar="", baz=""):
        return dict(title="Failsafe map errors", bar=str(bar), baz=str(baz))
    failsafemaperrors = validate(validators={"bar":validators.Int(not_empty=True),
        "baz":validators.Int(not_empty=True)}, failsafe_schema=FailsafeSchema.map_errors)(
        failsafemaperrors)
    failsafemaperrors = expose()(failsafemaperrors)

    def failsafeformencode(self, tg_errors=None, bar="", baz=""):
        return dict(title="Formencode if_invalid", bar=bar, baz=baz)
    failsafeformencode = validate(validators={"bar":validators.Int(
        if_invalid=1), "baz":validators.Int(if_invalid=2)})(failsafeformencode)
    failsafeformencode = expose()(failsafeformencode)

    def failsafedefaults(self, tg_errors=None, bar=1, baz=2):
        return dict(title="Failsafe map defaults", bar=bar, baz=baz)
    failsafedefaults = validate(validators={"bar":validators.Int(not_empty=True),
        "baz":validators.Int(not_empty=True)}, failsafe_schema=FailsafeSchema.defaults)(
        failsafedefaults)
    failsafedefaults = expose()(failsafedefaults)

class NestedController(Controller):

    def nest(self, bar=""):
        return dict(title="Nested")
    nest = error_handler()(nest)
    nest = validate(validators={"bar":validators.Int(not_empty=True)})(nest)
    nest = expose()(nest)


class TestErrorHandler(unittest.TestCase):

    def setUp(self):
        cherrypy.root = MyRoot()
        cherrypy.root.nestedcontroller = NestedController()

    def test_defaultErrorHandler(self):
        """ Default error handler. """
        testutil.createRequest("/defaulterror?bar=abc")
        self.failUnless("Default error handler" in cherrypy.response.body[0])
        testutil.createRequest("/defaulterror?bar=true")
        self.failUnless("Default error provider" in cherrypy.response.body[0])

    def test_specialisedErrorHandler(self):
        """ Error handler specialisation. """
        testutil.createRequest("/specialisederror?bar=abc&baz=a@b.com")
        self.failUnless("Default error handler" in cherrypy.response.body[0])
        testutil.createRequest("/specialisederror?baz=abc&bar=1")
        self.failUnless("Specialised error handler" in
                        cherrypy.response.body[0])
        testutil.createRequest("/specialisederror?bar=1&baz=a@b.com")
        self.failUnless("Specialised error provider" in
                        cherrypy.response.body[0])

    def test_exceptionErrorHandler(self):
        """ Error handler for exceptions. """
        testutil.createRequest("/exceptionerror")
        self.failUnless("Default error handler" in cherrypy.response.body[0])

    def test_recursiveErrorHandler(self):
        """ Recursive error handler. """
        testutil.createRequest("/recursiveerror?bar=abc")
        self.failUnless("Recursive error handler" in cherrypy.response.body[0])
        testutil.createRequest("/recursiveerror?bar=1")
        self.failUnless("Recursive error provider" in
                        cherrypy.response.body[0])

    def test_implicitErrorHandler(self):
        """ Implicit error handling. """
        testutil.createRequest("/impliciterror?bar=abc")
        self.failUnless("Implicit error handler" in
                        cherrypy.response.body[0])
        testutil.createRequest("/impliciterror?bar=1")
        self.failUnless("Implicit error provider" in
                        cherrypy.response.body[0])

    def test_normalMethodErrorHandler(self):
        """ Normal method as an error handler. """
        testutil.createRequest("/normalmethodcaller?bar=abc")
        self.failUnless("Normal method" in cherrypy.response.body[0])
        testutil.createRequest("/normalmethodcaller?bar=true")
        self.failUnless("Normal method caller" in cherrypy.response.body[0])

    def test_infiniteRecursionPrevention(self):
        """ Infinite recursion prevention. """
        testutil.createRequest("/infiniteloop")
        self.failUnless("Exception 2" in cherrypy.response.body[0])

    def test_positionalArgs(self):
        """ Positional argument validation.  """
        testutil.createRequest("/positionalargs/first/23/third?bar=abc")
        self.failUnless("Default error handler" in cherrypy.response.body[0])
        testutil.createRequest("/positionalargs/first/abc/third?bar=false")
        self.failUnless("Default error handler" in cherrypy.response.body[0])
        testutil.createRequest("/positionalargs/first/abc/third?bar=abc")
        self.failUnless("Default error handler" in cherrypy.response.body[0])
        testutil.createRequest("/positionalargs/first/23/third?bar=true")
        self.failUnless("Positional arguments" in cherrypy.response.body[0])
        self.failUnless(cherrypy.root.first == "first")
        self.failUnless(cherrypy.root.second == 23)
        self.failUnless(cherrypy.root.third == "third")

    def test_missingArgs(self):
        """ Arguments required in validation missing. """
        testutil.createRequest("/missingargs")
        self.failUnless("Default error handler" in cherrypy.response.body[0])
        testutil.createRequest("/missingargs?bar=12")
        self.failUnless("Missing args provider" in cherrypy.response.body[0])

    def test_nohandler(self):
        """ No error hanlder declared. """
        testutil.createRequest("/nohandler")
        self.failUnless("Exception raised" in cherrypy.response.body[0])

    def test_bindArgs(self):
        """ Arguments can be bond to an error handler. """
        testutil.createRequest("/bindargs")
        self.failUnless("123" in cherrypy.response.body[0])

    def test_notExposed(self):
        """ Validation error handling is decoupled from expose. """
        testutil.createRequest("/notexposedcaller?foo=a&bar=rab&baz=c")
        self.failUnless("Not exposed error" in cherrypy.response.body[0])
        self.failUnless("rab" in cherrypy.response.body[0])

    def test_continuations(self):
        """ Continuations via error handling mechanism. """
        testutil.createRequest("/continuationcaller?bar=a")
        self.failUnless("Continuation caller" in cherrypy.response.body[0])
        self.failUnless(cherrypy.root.continuation == True)

    def test_nested(self):
        """ Potentially ambiguous cases. """
        testutil.createRequest("/nest?bar=a")
        self.failUnless("Default error handler" in cherrypy.response.body[0])
        testutil.createRequest("/nestedcontroller/nest?bar=a")
        self.failUnless("Nested" in cherrypy.response.body[0])

    def test_failsafe(self):
        """ Failsafe values for erroneous input. """
        testutil.createRequest("/failsafenone?bar=a&baz=b")
        self.failUnless('"bar": "a"' in cherrypy.response.body[0])
        self.failUnless('"baz": "b"' in cherrypy.response.body[0])
        testutil.createRequest("/failsafevaluesdict?bar=a&baz=b")
        self.failUnless('"bar": 1' in cherrypy.response.body[0])
        self.failUnless('"baz": 2' in cherrypy.response.body[0])
        testutil.createRequest("/failsafevaluesatom?bar=a&baz=b")
        self.failUnless('"bar": 13' in cherrypy.response.body[0])
        self.failUnless('"baz": 13' in cherrypy.response.body[0])
        testutil.createRequest("/failsafemaperrors?bar=a&baz=b")
        self.failUnless('"bar": "Please enter an integer value"' in
                        cherrypy.response.body[0])
        self.failUnless('"baz": "Please enter an integer value"' in
                        cherrypy.response.body[0])
        testutil.createRequest("/failsafeformencode?bar=a&baz=b")
        self.failUnless('"bar": 1' in cherrypy.response.body[0])
        self.failUnless('"baz": 2' in cherrypy.response.body[0])
        testutil.createRequest("/failsafedefaults?bar=a&baz=b")
        self.failUnless('"bar": 1' in cherrypy.response.body[0],cherrypy.response.body[0])
        self.failUnless('"baz": 2' in cherrypy.response.body[0])



# coding=UTF-8
import re
import unittest
import cherrypy
import turbogears
from turbogears import testutil, database, identity, config, startup
from turbogears.identity.soprovider import TG_User, TG_Group, TG_Permission

#hub = database.AutoConnectHub("sqlite:///:memory:")
hub = database.PackageHub("turbogears.identity")

class RestrictedArea(turbogears.controllers.Controller, identity.SecureResource):

    require = identity.in_group('peon')

    def index(self):
        return "restricted_index"
    index = turbogears.expose()(index)

    def in_admin_group(self):
        return 'in_admin_group'
    in_admin_group = turbogears.expose()(in_admin_group)
    in_admin_group = identity.require(identity.in_group('admin'))(in_admin_group)

    def in_other_group(self):
        return 'in_other_group'
    in_other_group = turbogears.expose()(in_other_group)
    in_other_group = identity.require(identity.in_group('other'))(in_other_group)

    def in_admin_group_explicit_check(self):
        if 'admin' not in identity.current.groups:
            raise identity.IdentityFailure("You need to be an Admin")
        else:
            return 'in_admin_group'
    in_admin_group_explicit_check = turbogears.expose()(
        in_admin_group_explicit_check)

    def in_other_group_explicit_check(self):
        if 'other' not in identity.current.groups:
            raise identity.IdentityException
        else:
            return 'in_other_group'
    in_other_group_explicit_check = turbogears.expose()(
        in_other_group_explicit_check)

class IdentityRoot(turbogears.controllers.RootController):

    def index(self):
        pass
    index = turbogears.expose()(index)

    def identity_failed(self):
        return 'identity_failed_answer'
    identity_failed = turbogears.expose()(identity_failed)

    #@turbogears.expose()
    #@identity.require(identity.not_anonymous())
    def logged_in_only(self):
        return 'logged_in_only'
    logged_in_only = turbogears.expose()(logged_in_only)
    logged_in_only = identity.require(identity.not_anonymous())(logged_in_only)

    #@turbogears.expose()
    #@identity.require(identity.in_group('peon'))
    def in_peon_group(self):
        return 'in_peon_group'
    in_peon_group = turbogears.expose()(in_peon_group)
    in_peon_group = identity.require(identity.in_group('peon'))(in_peon_group)

    def test_exposed_require(self):
        if not hasattr(self.in_peon_group, '_require'):
            return 'no _require attr'
        if not isinstance(self.in_peon_group._require, identity.in_group):
            return 'not correct class'
        if 'peon' != self.in_peon_group._require.group_name:
            return 'not correct group name'
        return '_require is exposed'
    test_exposed_require= turbogears.expose()(test_exposed_require)

    def in_admin_group(self):
        return 'in_admin_group'
    in_admin_group = turbogears.expose()(in_admin_group)
    in_admin_group = identity.require(identity.in_group('admin'))(in_admin_group)

    def has_chopper_permission(self):
        return 'has_chopper_permission'
    has_chopper_permission = turbogears.expose()(has_chopper_permission)
    has_chopper_permission = identity.require(identity.has_permission('chops_wood'))(has_chopper_permission)

    def has_boss_permission(self):
        return "has_boss_permission"
    has_boss_permission = turbogears.expose()(has_boss_permission)
    has_boss_permission = identity.require(identity.has_permission('bosses_people'))(has_boss_permission)

    def logout(self):
        identity.current.logout()
        return "logged out"
    logout = turbogears.expose()(logout)

    def user_email(self):
        return identity.current.user.email_address
    user_email = turbogears.expose()(user_email)
    user_email = identity.require(identity.not_anonymous())(user_email)

    peon_area = RestrictedArea()

    def new_user_setup(self, user_name, password):
        return '%s %s' % (user_name, password)
    new_user_setup = turbogears.expose()(new_user_setup)


class TestIdentity(unittest.TestCase):

    def setUp(self):
        # identity requires visit and a failure_url
        test_config = {'visit.on': True, 'identity.on': True,
            'identity.failure_url': '/identity_failed',
            'identity.soprovider.encryption_algorithm': None}
        old_configuration = dict() 
        for key in test_config: 
            old_configuration[key] = config.get(key) 
        self._original_config = old_configuration         
        config.update(test_config)
        cherrypy.root = IdentityRoot()
        startup.startTurboGears()
        self.init_model()

    def tearDown(self):
        startup.stopTurboGears()
        config.update(self._original_config)

    def init_model(self):
        if TG_User.select().count() == 0:
            user = TG_User(user_name='samIam', email_address='samiam@example.com',
                            display_name='Samuel Amicus', password='secret')
            peon_group = TG_Group(group_name="peon", display_name="Regular Peon")
            admin_group = TG_Group(group_name="admin", display_name="Administrator")
            other_group = TG_Group(group_name="other", display_name="Another Group")
            chopper_perm = TG_Permission(permission_name='chops_wood', description="Wood Chopper")
            boss_perm = TG_Permission(permission_name='bosses_people', description="Benevolent Dictator")
            peon_group.addTG_Permission(chopper_perm)
            admin_group.addTG_Permission(boss_perm)
            user.addTG_Group(peon_group)
            user.addTG_Group(other_group)

    def test_user_password_parameters(self):
        "Controller can receive user_name and password parameters."
        testutil.create_request('/new_user_setup?user_name=test&password=pw')
        firstline = cherrypy.response.body[0]
        assert 'test pw' in firstline, firstline

    def test_user_exists(self):
        u = TG_User.by_user_name('samIam')
        assert u.email_address == 'samiam@example.com'

    def test_user_password(self):
        "Test if we can set a user password (no encryption algorithm)."
        hub.begin()
        u = TG_User.by_user_name('samIam')
        u.password='password'
        u.sync()
        assert u.password=='password'
        hub.rollback()
        hub.end()
        
    def test_user_password_unicode(self):
        """Test if we can set a user password which is encoded as unicode (no 
        encryption algorithm)."""
        config.update({'identity.soprovider.encryption_algorithm':None})
        # force new config values to load
        startup.startTurboGears()
        testutil.create_request('/')
        hub.begin()
        u = TG_User.by_user_name('samIam')
        u.password = u'garçon'
        u.sync()
        self.assertEqual(u.password, u'garçon')
        hub.rollback()
        hub.end()

    def test_user_password_hashed_sha(self):
        "Test if a sha hashed password is stored in the database."
        config.update({'identity.soprovider.encryption_algorithm':'sha1'})
        # force new config values to load
        startup.startTurboGears()
        testutil.create_request('/')
        hub.begin()
        u = TG_User.by_user_name('samIam')
        u.password = 'password'
        u.sync()
        assert u.password =='5baa61e4c9b93f3f0682250b6cf8331b7ee68fd8'
        hub.rollback()
        hub.end()
        
    def test_user_password_hashed_sha_unicode(self):
        """Test if a sha hashed password with unicode characters is stored in 
        the database."""
        config.update({'identity.soprovider.encryption_algorithm':'sha1'})
        # force new config values to load
        startup.startTurboGears()
        testutil.create_request('/')
        hub.begin()
        u = TG_User.by_user_name('samIam')
        u.password = u'garçon'
        u.sync()
        self.failUnlessEqual(u.password, '442edb21c491a6e6f502eb79e98614f3c7edf43e')
        hub.rollback()
        hub.end()

    def test_user_password_hashed_md5(self):
        "Test if a md5 hashed password is stored in the database."
        config.update({'identity.soprovider.encryption_algorithm':'md5'})
        # force new config values to load
        startup.startTurboGears()
        testutil.create_request('/')
        hub.begin()
        u = TG_User.by_user_name('samIam')
        u.password = 'password'
        u.sync()
        assert u.password =='5f4dcc3b5aa765d61d8327deb882cf99'
        hub.rollback()
        hub.end()
        
    def test_user_password_hashed_md5_unicode(self):
        """Test if a md5 hashed password with unicode characters is stored in 
        the database."""
        config.update({'identity.soprovider.encryption_algorithm':'md5'})
        # force new config values to load
        startup.startTurboGears()
        testutil.create_request('/')
        hub.begin()
        u = TG_User.by_user_name('samIam')
        u.password = u'garçon'
        u.sync()
        self.assertEqual(u.password, 'c295c4bb2672ca8c432effc53b40bb1e')
        hub.rollback()
        hub.end()

    def test_user_password_hashed_md5_utf8string(self):
        """Test if a md5 hashed password with unicode characters is stored in 
        the database if the password is entered as str (encoded in UTF-8). This
        test ensures that the encryption algorithm does handle non-unicode
        parameters gracefully."""
        config.update({'identity.soprovider.encryption_algorithm':'md5'})
        # force new config values to load
        startup.startTurboGears()
        testutil.create_request('/')
        hub.begin()
        u = TG_User.by_user_name('samIam')
        u.password = u'garçon'.encode('UTF-8')
        u.sync()
        self.assertEqual(u.password, 'c295c4bb2672ca8c432effc53b40bb1e')
        hub.rollback()
        hub.end()

    def test_user_password_raw(self):
        "Test that we can store raw values in the password field (without being hashed)."
        config.update({'identity.soprovider.encryption_algorithm':'sha1'})
        # force new config values to load
        startup.startTurboGears()
        testutil.create_request('/')
        hub.begin()
        u = TG_User.by_user_name('samIam')
        u.set_password_raw('password')
        u.sync()
        assert u.password =='password'
        hub.rollback()
        hub.end()
        
    def test_user_password_raw_unicode(self):
        config.update({'identity.soprovider.encryption_algorithm':'sha1'})
        # force new config values to load
        startup.startTurboGears()
        testutil.create_request('/')
        hub.begin()
        u = TG_User.by_user_name('samIam')
        u.set_password_raw(u'garçon')
        u.sync()
        self.assertEqual(u.password, u'garçon')
        hub.rollback()
        hub.end()
        
    def test_anonymous_browsing(self):
        "Test if we can show up anonymously."
        testutil.create_request('/')
        assert identity.current.anonymous

    def test_deny_anonymous(self):
        "Test that we have secured an url from anonymous users."
        testutil.create_request('/logged_in_only')
        firstline = cherrypy.response.body[0]
        assert 'identity_failed_answer' in firstline, firstline

    def test_deny_anonymous_viewable(self):
        "Test that a logged in user can see an resource blocked from anonymous users."
        testutil.create_request('/logged_in_only?user_name=samIam&password=secret&login=Login')
        firstline = cherrypy.response.body[0]
        assert 'logged_in_only' in firstline, firstline

    def test_logout(self):
        "Test that logout works and the session is not valid anymore afterwards."
        testutil.create_request('/in_peon_group?user_name=samIam&password=secret&login=Login')
        self.assertEquals("samIam", cherrypy.serving.request.identity.user_name)

        x = re.match("Set-Cookie: (.*?); Path.*", str(cherrypy.response.simple_cookie) )
        session_id = x.group(1)
        testutil.create_request('/logout', headers={'Cookie': session_id })

        self.assertEquals(None, cherrypy.serving.request.identity.user_name)
        assert cherrypy.serving.request.identity.anonymous

    def test_logout_with_set_identity(self):
        """Test that logout works even when there is no visit_key (e.g. when 
        testutils.set_identity_user is used)."""
        request = testutil.DummyRequest()
        old_user = testutil.test_user
        user = TG_User.by_user_name("samIam")
        testutil.set_identity_user(user)
        testutil.attach_identity(request)
        testutil.set_identity_user(old_user)
        testutil.call_with_request(cherrypy.root.logout, request)
        assert request.identity.anonymous

    def test_require_group(self):
        "Test that a anonymous user"
        testutil.create_request('/in_peon_group')
        firstline = cherrypy.response.body[0]
        assert 'identity_failed_answer' in firstline, firstline

    def test_require_expose_required_permission(self):
        '''
        Test that the decorator exposes the correct permissions via _require
        attribute on the actual method.
        '''
        testutil.create_request('/test_exposed_require')
        firstline= cherrypy.response.body[0]
        assert 'require is exposed' in firstline, firstline

    def test_require_group_viewable(self):
        "Test that a user with proper group membership can see a restricted url."
        testutil.create_request('/in_peon_group?user_name=samIam&password=secret&login=Login')
        firstline = cherrypy.response.body[0]
        assert 'in_peon_group' in firstline, firstline

    def test_user_not_in_right_group(self):
        "Test that a user is denied access if they aren't in the right group."
        testutil.create_request('/in_admin_group?user_name=samIam&password=secret&login=Login')
        firstline = cherrypy.response.body[0]
        assert 'identity_failed_answer' in firstline, firstline

    def test_require_permission(self):
        "Test that an anonymous user is denied access to a permission restricted url."
        testutil.create_request('/has_chopper_permission')
        firstline = cherrypy.response.body[0]
        assert 'identity_failed_answer' in firstline, firstline

    def test_require_permission_viewable(self):
        "Test that a user with proper permissions can see a restricted url."
        testutil.create_request('/has_chopper_permission?user_name=samIam&password=secret&login=Login')
        firstline = cherrypy.response.body[0]
        assert 'has_chopper_permission' in firstline, firstline

    def test_user_lacks_permission(self):
        "Test that a user is denied acces if they don't have the proper permission."
        testutil.create_request('/has_boss_permission?user_name=samIam&password=secret&login=Login')
        firstline = cherrypy.response.body[0]
        assert 'identity_failed_answer' in firstline, firstline

    def test_user_info_available(self):
        "Test that we can see user information inside our controller."
        testutil.create_request('/user_email?user_name=samIam&password=secret&login=Login')
        firstline = cherrypy.response.body[0]
        assert 'samiam@example.com' in firstline, firstline

    def test_bad_login(self):
        "Test that we are denied access if we provide a bad login."
        testutil.create_request('/logged_in_only?user_name=samIam&password=wrong&login=Login')
        firstline = cherrypy.response.body[0]
        assert 'identity_failed_answer' in firstline, firstline

    def test_restricted_subdirectory(self):
        "Test that we can restrict access to a whole subdirectory."
        testutil.create_request('/peon_area/index')
        firstline = cherrypy.response.body[0]
        assert 'identity_failed_answer' in firstline, firstline

    def test_restricted_subdirectory_viewable(self):
        "Test that we can access a restricted subdirectory if we have proper credentials."
        testutil.create_request('/peon_area/index?user_name=samIam&password=secret&login=Login')
        firstline = cherrypy.response.body[0]
        assert 'restricted_index' in firstline, firstline

    def test_decoratator_in_restricted_subdirectory(self):
        """Test that we can require a different permission in a
        protected subdirectory."""
        testutil.create_request('/peon_area/in_other_group?user_name=samIam&password=secret&login=Login')
        firstline = cherrypy.response.body[0]
        assert 'in_other_group' in firstline, firstline

    def test_decoratator_failure_in_restricted_subdirectory(self):
        """Test that we can get an identity failure from a decorator
        in a restricted subdirectory"""
        testutil.create_request('/peon_area/in_admin_group?user_name=samIam&password=secret&login=Login')
        firstline = cherrypy.response.body[0]
        assert 'identity_failed_answer' in firstline, firstline

    def test_explicit_checks_in_restricted_subdirectory(self):
        """Test that explicit permission checks in a protected
        directory is handled as expected"""
        testutil.create_request('/peon_area/in_other_group_explicit_check?user_name=samIam&password=secret&login=Login')
        firstline = cherrypy.response.body[0]
        assert 'in_other_group' in firstline, firstline

    def test_throwing_identity_exception_in_restricted_subdirectory(self):
        """Test that throwing an IdentityException in a protected
        directory is handled as expected"""
        testutil.create_request('/peon_area/in_admin_group_explicit_check?user_name=samIam&password=secret&login=Login')
        firstline = cherrypy.response.body[0]
        assert 'identity_failed' in firstline, firstline


class TestTGUser(testutil.DBTest):
    model = TG_User

    def setUp(self):
        self._identity_on = config.get('identity.on', False)
        config.update({'identity.on': False})
        try:
            self._provider = cherrypy.request.identityProvider
        except AttributeError:
            self._provider= None
        cherrypy.request.identityProvider = None
        startup.startTurboGears()
        testutil.DBTest.setUp(self)

    def tearDown(self):
        testutil.DBTest.tearDown(self)
        startup.stopTurboGears()
        cherrypy.request.identityProvider = self._provider
        config.update({'identity.on': self._identity_on})

    def test_create_user(self):
        """Check that User can be created outside of a running identity provider."""
        u = TG_User(user_name='testcase', email_address='testcase@example.com',
                        display_name='Test Me', password='test')
        assert u.password=='test', u.password



"""Quickstart command to generate a new project.

Quickstart takes the files from turbogears.quickstart and processes them to produce
a new, ready-to-run project."""

import pkg_resources
import re
import os
import os.path
import shutil
import stat
import optparse
import sys
import turbogears
from paste.script import templates, create_distro

beginning_letter = re.compile(r"^[^a-z]*")
valid_only = re.compile(r"[^a-z0-9_]")

class TGTemplate(templates.Template):
    def run(self, command, output_dirs, vars):
        vars.setdefault('einame', vars['project'].replace('-', '_'))
        vars.setdefault('turbogearsversion',
                pkg_resources.get_distribution('turbogears').version)
        vars.setdefault('sys_executable', os.path.normpath(sys.executable))
        # define version-compatible decorator syntax
        if sys.version_info >= (2, 4):
            vars['b'] = '@'
            vars['e'] = ''
        else:
            vars['b'] = '['
            vars['e'] = ']'
        super(TGTemplate, self).run(command, output_dirs, vars)

class BaseTemplate(TGTemplate):
    egg_plugins = ["TurboGears"]
    _template_dir = pkg_resources.resource_filename(
                        "turbogears.qstemplates",
                        "qsbase"
                    )
    summary = "tg base template"

class TurbogearsTemplate(TGTemplate):

    required_templates = ["tgbase"]
    _template_dir = pkg_resources.resource_filename(
                            "turbogears.qstemplates", 
                            "quickstart")
    summary = "web framework"
    use_cheetah = True
   

class TGBig(TGTemplate):
    required_templates = ["turbogears"]
    _template_dir = pkg_resources.resource_filename(
                            "turbogears.qstemplates", 
                            "quickstartbig")
    summary = "For more complex projects"

    def post(self, command, output_dir, vars):
        packagedir = os.path.join(output_dir, vars["package"])
        controllersdir = os.path.join(packagedir, "controllers")
        controllersfile = os.path.join(packagedir, "controllers.py")
        rootfile = os.path.join(controllersdir, "root.py")
        if os.path.exists(controllersdir) and \
            os.path.exists(controllersfile):
            controllerstext = open(controllersfile).read()
            roottext = open(rootfile).read()
            from paste.script.copydir import query_interactive
            thesame = controllerstext == roottext
            if not thesame:
                print "\nYou seem to be upgrading from a smaller quickstart."
                print "There is currently a controllers package and a"
                print "controllers module, which would get confusing."
            if controllerstext == roottext or \
                    query_interactive(controllersfile, rootfile,
                    controllerstext, roottext, False):
                shutil.copyfile(controllersfile, rootfile)
                os.unlink(controllersfile)
                controllerspyc = controllersfile + "c"
                if os.path.exists(controllerspyc):
                    os.unlink(controllerspyc)

class TGWidgetTemplate(TGTemplate):
    required_templates = ["tgbase"]
    _template_dir = pkg_resources.resource_filename(
                            "turbogears.qstemplates", 
                            "widget")
    summary = "TurboGears widget projects"

class quickstart:
    "Implementation of quickstart."

    desc = "Create a new TurboGears project"
    
    name = None
    package = None
    templates = "turbogears"
    sqlalchemy = False
    identity = False
    
    def __init__(self, version):
        parser = optparse.OptionParser(
                    usage="%prog quickstart [options] [project name]",
                    version="%prog " + version)
        parser.add_option("-p", "--package", 
            help="package name for the code",
            dest="package")
        parser.add_option("--dry-run", 
            help="dry run (don't actually do anything)",
            action="store_true", dest="dry_run")
        parser.add_option("-t", "--templates", 
            help="user specific templates",
            dest="templates", default = self.templates)
        parser.add_option("-s", "--sqlalchemy",
            help="use SQLAlchemy instead of SQLObject",
            action="store_true", dest="sqlalchemy", default = False)
        parser.add_option("-i", "--identity",
            help="provide Identity support",
            action="store_true", dest="identity", default = False)

        (options, args) = parser.parse_args()
        self.__dict__.update(options.__dict__)
        if args:
            self.name = args[0]
        self.turbogearsversion = version
    
    def run(self):
        "Quickstarts the new project."
        
        while not self.name:
            self.name = raw_input("Enter project name: ")
        
        while not self.package:
            package = self.name.lower()
            package = beginning_letter.sub("", package)
            package = valid_only.sub("", package)
            self.package = raw_input("Enter package name [%s]: " % package)
            if not self.package:
                self.package = package

        doidentity = self.identity
        while not doidentity:
            doidentity = raw_input("Do you need Identity "
                        "(usernames/passwords) in this project? [no] ")
            doidentity = doidentity.lower()
            if not doidentity or doidentity.startswith('n'): 
                self.identity="none"
                break
            if doidentity.startswith("y"):
                doidentity = True
                break
            print "Please enter y(es) or n(o)."
            doidentity = None
        
        if doidentity is True:
            if self.sqlalchemy:
                self.identity = "sqlalchemy"
            else:
                self.identity = "sqlobject"
        self.name = pkg_resources.safe_name(self.name)

        env = pkg_resources.Environment()
        if self.name.lower() in env:
            print "the name %s is already in use by" %self.name,
            for dist in env[self.name]:
                print dist
                return

        import imp
        try:
            if imp.find_module(self.package):
                print "the package name %s is already in use" % self.package
                return
        except ImportError:
            pass

        if os.path.exists(self.name):
            print("A directory called '%s' already exists. Exiting."
                      % self.name)
            return        
            
        
        command = create_distro.CreateDistroCommand("create")
        cmd_args = []
        for template in self.templates.split(" "):
            cmd_args.append("--template=%s" % template)
        cmd_args.append(self.name)
        cmd_args.append("package=%s" % self.package)
        cmd_args.append("identity=%s" % self.identity)
        cmd_args.append("sqlalchemy=%s" % self.sqlalchemy)
        if self.dry_run:
            cmd_args.append("--simulate")
            cmd_args.append("-q")
        command.run(cmd_args)
        
        if not self.dry_run:
            os.chdir(self.name)
            startscript = "start-%s.py" % self.package
            if os.path.exists(startscript):
                oldmode = os.stat(startscript).st_mode
                os.chmod(startscript, 
                        oldmode | stat.S_IXUSR)
            sys.argv = ["setup.py", "egg_info"]
            import imp
            imp.load_module("setup", *imp.find_module("setup", ["."]))
            
            # dirty hack to allow "empty" dirs
            for base,path,files in os.walk("./"):
                for file in files:
                    if file  == "empty":
                        os.remove(os.path.join(base, file))

class update:
    "Implementation of update"
    
    desc = "Update an existing turbogears project"
    need_project = True
    
    name = None
    templates = "turbogears"
    identity = False
    sqlalchemy = False
    
    def __init__(self, version):
        parser = optparse.OptionParser(usage="%prog quickstart [options]",
                                       version="%prog " + version)
        parser.add_option("-t", "--templates", help="user specific templates",
            dest="templates", default=self.templates)
        parser.add_option("-s", "--sqlalchemy",
            help="use SQLAlchemy instead of SQLObject",
            action="store_true", dest="sqlalchemy", default = False)
        parser.add_option("-i", "--identity",
            help="provide Identity support",
            action="store_true", dest="identity", default = False)
        (options, args) = parser.parse_args()
        self.__dict__.update(options.__dict__)
        self.turbogearsversion = version

    def run(self):
        "Updates an existing project"
        self.name = turbogears.util.get_project_name()
        self.package = turbogears.util.get_package_name()
        turbogears.command.base.load_project_config()
        if not self.sqlalchemy:
            if turbogears.config.get('sqlalchemy.dburi'):
                self.sqlalchemy = True 
        if not self.identity:
            if turbogears.config.get('identity.on'):
                self.identity = True
        if self.identity:
            if self.sqlalchemy:
                self.identity = 'sqlalchemy'
            else:
                self.identity =  'sqlobject'
        else:
            self.identity = 'none'
        currentdir = os.path.basename(os.getcwd())
        if not currentdir == self.name:
            print 'it looks like your project dir "%s" is named wrongly. Please rename it to "%s"' %(currentdir, self.name)
            return
        
        command = create_distro.CreateDistroCommand("create")
        cmd_args = []
        cmd_args.append("-o../")
        for template in self.templates.split(" "):
            cmd_args.append("--template=%s" % template)
        cmd_args.append(self.name)
        cmd_args.append("package=%s" %self.package)
        cmd_args.append("identity=%s" %self.identity)
        cmd_args.append("sqlalchemy=%s" %self.sqlalchemy)
        command.run(cmd_args)

        startscript = "start-%s.py" % self.package
        if os.path.exists(startscript):
            oldmode = os.stat(startscript).st_mode
            os.chmod(startscript, 
                    oldmode | stat.S_IXUSR)
        sys.argv = ["setup.py", "egg_info"]
        import imp
        imp.load_module("setup", *imp.find_module("setup", ["."]))
        
        # dirty hack to allow "empty" dirs
        for base,path,files in os.walk("./"):
            for file in files:
                if file  == "empty":
                    os.remove(os.path.join(base, file))




"""Template processing for TurboGears view templates."""

import os
import imp
import sys
import re
import logging
from itertools import chain
from itertools import cycle as icycle
from urllib import quote_plus

import cherrypy
import pkg_resources

import turbogears
from turbogears import identity, config
from turbogears.i18n import i18n_filter, get_locale
from turbogears.util import DictObj, DictWrapper, get_template_encoding_default, adapt_call

log = logging.getLogger("turbogears.view")

baseTemplates = []
variable_providers = []
root_variable_providers = []
engines = dict()

# Deprecation of variableProviders
def print_warning(func):
    def _print_warning(*args, **kw):
        import warnings
        warnings.warn(
            "Use of variableProviders is deprecated, use variable_providers"
            "instead.",
            DeprecationWarning, 2)
        return func(*args, **kw)
    return _print_warning

class MetaDeprecatedVariableProviders(type):
    def __new__(cls, name, bases, dict):
        for (key, value) in dict.items():
            if key == "__metaclass__":
                continue
            if callable(value):
                dict[key] = print_warning(value)
        return type.__new__(cls,name, bases, dict)

class DeprecatedVariableProviders(list):
    __metaclass__ = MetaDeprecatedVariableProviders

    def append(self, *args, **kw):
        super(DeprecatedVariableProviders, self).append(*args, **kw)

    def count(self, *args, **kw):
        return super(DeprecatedVariableProviders, self).count(*args, **kw)

    def extend(self, *args, **kw):
        super(DeprecatedVariableProviders, self).extend(*args, **kw)

    def index(self, *args, **kw):
        return super(DeprecatedVariableProviders, self).index(*args, **kw)

    def insert(self, *args, **kw):
        super(DeprecatedVariableProviders, self).insert(*args, **kw)

    def pop(self, *args, **kw):
        return super(DeprecatedVariableProviders, self).pop(*args, **kw)

    def remove(self, *args, **kw):
        super(DeprecatedVariableProviders, self).remove(*args, **kw)

    def reverse(self, *args, **kw):
        super(DeprecatedVariableProviders, self).reverse(*args, **kw)

    def sort(self, *args, **kw):
        super(DeprecatedVariableProviders, self).sort(*args, **kw)

variableProviders = DeprecatedVariableProviders()

def _choose_engine(template):
    if isinstance(template, basestring):
        colon = template.find(":")
        if colon > -1:
            enginename = template[:colon]
            template = template[colon+1:]
        else:
            engine = engines.get(template, None)
            if engine:
                return engine, None, template
            enginename = config.get("tg.defaultview", "kid")
    else:
        enginename = config.get("tg.defaultview", "kid")
    engine = engines.get(enginename, None)
    if not engine:
        raise KeyError, \
            "Template engine %s is not installed" % enginename
    return engine, template, enginename


def render(info, template=None, format=None ,content_type=None, mapping=None, fragment=False):
    """Renders data in the desired format.

    @param info: the data itself
    @type info: dict
    @param format: "html", "xml" or "json"
    @type format: string
    @param fragment: passed through to tell the template if only a
                     fragment of a page is desired
    @type fragment: bool
    @param template: name of the template to use
    @type template: string
    """
    template = info.pop("tg_template", template)
    if not info.has_key("tg_flash"):
        if config.get("tg.empty_flash", True):
            info["tg_flash"] = None
    engine, template, enginename = _choose_engine(template)
    if  not content_type and getattr(engine, 'get_content_type', None):
        ua = getattr(cherrypy.request.headers, "User-Agent", None)
        ua = UserAgent(ua)
        content_type = engine.get_content_type(ua)
    elif not content_type:
        content_type = "text/html"
    if content_type == 'text/html' and enginename in ('genshi', 'kid'):
        charset = get_template_encoding_default(enginename)
        content_type = content_type + '; charset=' + charset
    cherrypy.response.headers["Content-Type"] = content_type
    if not format:
        format = config.get("%s.outputformat" % enginename, "html") 
    args, kw = adapt_call(engine.render, args= [],
                kw = dict(info=info, format=format, fragment=fragment, template=template, mapping=mapping), start=1)
    return engine.render(**kw)

def transform(info, template):
    "Create ElementTree representation of the output"
    engine, template, enginename = _choose_engine(template)
    return engine.transform(info, template)

def loadBaseTemplates():
    """Loads base templates for use by other templates.
    By listing templates in turbogears.view.baseTemplates, these templates will
    automatically be loaded so that the "import" statement in a template
    will work.
    """
    log.debug("Loading base templates")
    for template in baseTemplates:
        engine, template, enginename = _choose_engine(template)
        if sys.modules.has_key(template):
            del sys.modules[template]
        engine.load_template(template)

NOTGIVEN = []

class cycle:
    """
    Loops forever over an iterator. Wraps the itertools.cycle method
    but provides a way to get the current value via the 'value' attribute

    >>> from turbogears.view.base import cycle
    >>> oe = cycle(('odd','even'))
    >>> oe
    None
    >>> oe.next()
    'odd'
    >>> oe
    'odd'
    >>> oe.next()
    'even'
    >>> oe.next()
    'odd'
    >>> oe.value
    'odd'
    """
    value = None
    def __init__(self,iterable):
        self._cycle = icycle(iterable)
    def __str__(self):
        return self.value.__str__()
    def __repr__(self):
        return self.value.__repr__()
    def next(self):
        self.value = self._cycle.next()
        return self.value


def selector(expression):
    """If the expression is true, return the string 'selected'. Useful for
    HTML <option>s."""
    if expression:
        return "selected"
    else:
        return None

def checker(expression):
    """If the expression is true, return the string "checked". This is useful
    for checkbox inputs.
    """
    if expression:
        return "checked"
    else:
        return None

def ipeek(it):
    """Lets you look at the first item in an iterator. This is a good way
    to verify that the iterator actually contains something. This is useful
    for cases where you will choose not to display a list or table if there
    is no data present.
    """
    it = iter(it)
    try:
        item = it.next()
        return chain([item], it)
    except StopIteration:
        return None

safariua = re.compile(r"Safari/(\d+)")

class UserAgent:
    """Representation of the user's browser.

    Provides information about the type of browser, browser version,
    etc. This currently contains only the information needed for work
    thus far. (msie, firefox, safari browser types, plus safari version
    info.)
    """
    def __init__(self, useragent=None):
        self.majorVersion = None
        self.minorVersion = None
        if not useragent:
            useragent = "unknown"
        if useragent.find("MSIE") > -1:
            self.browser = "msie"
        elif useragent.find("Firefox") > -1:
            self.browser = "firefox"
        else:
            isSafari = safariua.search(useragent)
            if isSafari:
                self.browser = "safari"
                build = int(isSafari.group(1))
                # this comes from:
                # http://developer.apple.com/internet/safari/uamatrix.html
                if build >= 412:
                    self.majorVersion = "2"
                    self.minorVersion = "0"
                elif build >= 312:
                    self.majorVersion = "1"
                    self.minorVersion = "3"
                elif build >= 125:
                    self.majorVersion = "1"
                    self.minorVersion = "2"
                elif build >= 85:
                    self.majorVersion = "1"
                    self.minorVersion = "0"
            elif useragent == "unknown" or useragent is None:
                self.browser = "unknown"
            else:
                self.browser = "unknown: %s" % useragent

class DeprecatedDictWrapper(DictWrapper):
    """Wraps access with a deprecation warning."""

    def __getattr__(self,name):
        import warnings
        warnings.warn(
            "The use of 'std' inside your templates is deprecated,"
            " use 'tg' instead.",
            DeprecationWarning, 2)
        return super(DeprecatedDictWrapper, self).__getattr__(name)

def stdvars():
    """Creates an DictObj with variables that should be available in all
    templates. These variables are:

    useragent
        a UserAgent object with information about the browser
    selector
        the selector function
    checker
        the checker function
    tg_js
        the path to the JavaScript libraries
    ipeek
        the ipeek function
    cycle
        cycle through a set of values
    quote_plus
        the urllib quote_plus function
    url
        the turbogears.url function for creating flexible URLs
    identity
        the current visitor's identity information
    locale
        the default locale
    inputs
        input values from a form
    errors
        validation errors
    request
        the cherrypy request
    config
        the cherrypy config get function

    Additionally, you can add a callable to turbogears.view.variable_providers
    that can add more variables to this list. The callable will be
    called with the vars DictObj after these standard variables have
    been set up.
    """
    try:
        useragent = cherrypy.request.headers["User-Agent"]
        useragent = UserAgent(useragent)
    except:
        useragent = UserAgent()
    vars = DictObj(
        useragent=useragent, selector=selector,
        tg_js="/" + turbogears.startup.webpath + "tg_js",
        tg_toolbox="/" + turbogears.startup.webpath + "tg_toolbox",
        widgets="/" + turbogears.startup.webpath + "tg_widgets",
        tg_static="/" + turbogears.startup.webpath + "tg_static",
        ipeek=ipeek, cycle=cycle, quote_plus=quote_plus, checker=checker,
        url = turbogears.url, identity=identity.current, config=config.get,
        locale = get_locale(),
        errors = getattr(cherrypy.request, "validation_errors", {}),
        inputs = getattr(cherrypy.request, "input_values", {}),
        request = cherrypy.request)
    for provider in variable_providers + variableProviders:
        provider(vars)
    deprecated_vars = DeprecatedDictWrapper(vars)
    root_vars = dict()
    for provider in root_variable_providers:
        provider(root_vars)
    root_vars.update(dict(tg=vars, std=deprecated_vars))
    return root_vars

def load_engines():
    config = turbogears.config
    engine_options ={
        "mako.directories" : config.get("mako.directories", []),
        "mako.output_encoding" : config.get("mako.output_encoding", "utf-8"),
        "genshi.encoding" : config.get("genshi.encoding", "utf-8"),
        "genshi.default_doctype" : config.get("genshi.default_doctype", None),
        "genshi.variable_lookup" : config.get("genshi.variable_lookup", "strict"),
        "kid.encoding" : config.get("kid.encoding", "utf-8"),
        "kid.assume_encoding" : config.get("kid.assume_encoding", "utf-8"),
        "kid.precompiled" : config.get("kid.precompiled", False),
        "kid.i18n.run_template_filter" : config.get("i18n.run_template_filter",
                                                  False),
        "kid.i18n_filter" : i18n_filter,
        "kid.sitetemplate" : config.get("tg.sitetemplate",
                            "turbogears.view.templates.sitetemplate"),
        "kid.reloadbases" : config.get("kid.reloadbases", False)
    }
    for entrypoint in pkg_resources.iter_entry_points(
                                            "python.templating.engines"):
        engine = entrypoint.load()
        engines[entrypoint.name] = engine(stdvars, engine_options)
load_engines()



"""Provides convenient access to an SQLObject-managed database."""

import sys
import time
import logging

try:
    import sqlobject
    from sqlobject.dbconnection import ConnectionHub, Transaction, TheURIOpener
    from sqlobject.util.threadinglocal import local as threading_local
except ImportError:
    sqlobject = None

import cherrypy
from cherrypy.filters.basefilter import BaseFilter

import dispatch
from turbogears import config
from turbogears.util import remove_keys
from turbogears.genericfunctions import MultiorderGenericFunction

log = logging.getLogger("turbogears.database")

_engine = None

# Provide support for sqlalchemy
try:
    import sqlalchemy
    from sqlalchemy.ext import activemapper, sessioncontext

    def get_engine():
        "Retreives the engine based on the current configuration"
        global _engine
        if not _engine:
            alch_args = dict()
            for k, v in config.config.configMap["global"].items():
                if "sqlalchemy" in k:
                    alch_args[k.split(".")[-1]] = v
            dburi = alch_args.pop('dburi')
            if not dburi:
                raise KeyError("No sqlalchemy database config found!")
            _engine = sqlalchemy.create_engine(dburi, **alch_args)
            metadata.connect(_engine)
        elif not metadata.is_bound():
            metadata.connect(_engine)
        return _engine

    def create_session():
        "Creates a session with the appropriate engine"
        return sqlalchemy.create_session(bind_to=get_engine())

    metadata = activemapper.metadata
    session = activemapper.Objectstore(create_session)
    activemapper.objectstore = session

    def bind_meta_data():
        get_engine()

except ImportError:
    sqlalchemy = None


try:
    set
except NameError:
    from sets import Set as set

hub_registry = set()

# This dictionary stores the AutoConnectHubs used for each
# connection URI
_hubs = dict()


if sqlobject:
    def _mysql_timestamp_converter(raw):
        """Convert a MySQL TIMESTAMP to a floating point number representing
        the seconds since the Un*x Epoch. It uses custom code the input seems 
        to be the new (MySQL 4.1+) timestamp format, otherwise code from the 
        MySQLdb module is used."""
        if raw[4] == '-':
            return time.mktime(time.strptime(raw, '%Y-%m-%d %H:%M:%S'))
        else:
            import MySQLdb.converters
            return MySQLdb.converters.mysql_timestamp_converter(raw)
            

    class AutoConnectHub(ConnectionHub):
        """Connects to the database once per thread. The AutoConnectHub also
        provides convenient methods for managing transactions."""
        uri = None
        params = {}

        def __init__(self, uri=None, supports_transactions=True):
            if not uri:
                uri = config.get("sqlobject.dburi")
            self.uri = uri
            self.supports_transactions = supports_transactions
            hub_registry.add(self)
            ConnectionHub.__init__(self)

        def _is_interesting_version(self):
            "Return True only if version of MySQLdb <= 1.0."
            import MySQLdb
            module_version = MySQLdb.version_info[0:2]
            major = module_version[0]
            minor = module_version[1]
            # we can't use Decimal here because it is only available for Python 2.4
            return (major < 1 or (major == 1 and minor < 2))

        def _enable_timestamp_workaround(self, connection):
            """Enable a workaround for an incompatible timestamp format change 
            in MySQL 4.1 when using an old version of MySQLdb. See trac ticket 
            #1235 - http://trac.turbogears.org/ticket/1235 for details."""
            # precondition: connection is a MySQLConnection
            import MySQLdb
            import MySQLdb.converters
            if self._is_interesting_version():
                conversions = MySQLdb.converters.conversions.copy()
                conversions[MySQLdb.constants.FIELD_TYPE.TIMESTAMP] = \
                    _mysql_timestamp_converter
                # There is no method to use custom keywords when using 
                # "connectionForURI" in sqlobject so we have to insert the
                # conversions afterwards.
                connection.kw["conv"] = conversions

        def getConnection(self):
            try:
                conn = self.threadingLocal.connection
                return self.begin(conn)
            except AttributeError:
                if self.uri:
                    conn = sqlobject.connectionForURI(self.uri)
                    # the following line effectively turns off the DBAPI connection
                    # cache. We're already holding on to a connection per thread,
                    # and the cache causes problems with sqlite.
                    if self.uri.startswith("sqlite"):
                        TheURIOpener.cachedURIs = {}
                    elif self.uri.startswith("mysql") and \
                         config.get("turbogears.enable_mysql41_timestamp_workaround", False):
                        self._enable_timestamp_workaround(conn)
                    self.threadingLocal.connection = conn
                    return self.begin(conn)
                raise AttributeError(
                    "No connection has been defined for this thread "
                    "or process")

        def reset(self):
            """Used for testing purposes. This drops all of the connections
            that are being held."""
            self.threadingLocal = threading_local()

        def begin(self, conn=None):
            "Starts a transaction."
            if not self.supports_transactions:
                return conn
            if not conn:
                conn = self.getConnection()
            if isinstance(conn, Transaction):
                if conn._obsolete:
                    conn.begin()
                return conn
            self.threadingLocal.old_conn = conn
            trans = conn.transaction()
            self.threadingLocal.connection = trans
            return trans

        def commit(self):
            "Commits the current transaction."
            if not self.supports_transactions:
                return
            try:
                conn = self.threadingLocal.connection
            except AttributeError:
                return
            if isinstance(conn, Transaction):
                self.threadingLocal.connection.commit()

        def rollback(self):
            "Rolls back the current transaction."
            if not self.supports_transactions:
                return
            try:
                conn = self.threadingLocal.connection
            except AttributeError:
                return
            if isinstance(conn, Transaction) and not conn._obsolete:
                self.threadingLocal.connection.rollback()

        def end(self):
            "Ends the transaction, returning to a standard connection."
            if not self.supports_transactions:
                return
            try:
                conn = self.threadingLocal.connection
            except AttributeError:
                return
            if not isinstance(conn, Transaction):
                return
            if not conn._obsolete:
                conn.rollback()
            self.threadingLocal.connection = self.threadingLocal.old_conn
            del self.threadingLocal.old_conn
            self.threadingLocal.connection.cache.clear()

    class PackageHub(object):
        """Transparently proxies to an AutoConnectHub for the URI
        that is appropriate for this package. A package URI is
        configured via "packagename.dburi" in the global CherryPy
        settings. If there is no package DB URI configured, the
        default (provided by "sqlobject.dburi") is used.

        The hub is not instantiated until an attempt is made to
        use the database.
        """
        def __init__(self, packagename):
            self.packagename = packagename
            self.hub = None

        def __get__(self, obj, type):
            if not self.hub:
                self.set_hub()
            return self.hub.__get__(obj, type)

        def __set__(self, obj, type):
            if not self.hub:
                self.set_hub()
            return self.hub.__set__(obj, type)

        def __getattr__(self, name):
            if not self.hub:
                self.set_hub()
            return getattr(self.hub, name)

        def set_hub(self):
            dburi = config.get("%s.dburi" % self.packagename, None)
            if not dburi:
                dburi = config.get("sqlobject.dburi", None)
            if not dburi:
                raise KeyError, "No database configuration found!"
            if dburi.startswith("notrans_"):
                dburi = dburi[8:]
                trans = False
            else:
                trans = True
            hub = _hubs.get(dburi, None)
            if not hub:
                hub = AutoConnectHub(dburi, supports_transactions=trans)
                _hubs[dburi] = hub
            self.hub = hub
else:
    class AutoConnectHub(object):
        pass
    
    class PackageHub(object):
        pass

def set_db_uri(dburi, package=None):
    """Sets the database URI to use either globally or for a specific
    package. Note that once the database is accessed, calling
    setDBUri will have no effect.

    @param dburi: database URI to use
    @param package: package name this applies to, or None to set the default.
    """
    if package:
        config.update({'global':
            {"%s.dburi" % package : dburi}
        })
    else:
        config.update({'global':
            {"sqlobject.dburi" : dburi}
        })

def commit_all():
    "Commits the Transactions in all registered hubs (for this thread)"
    for hub in hub_registry:
        hub.commit()

def rollback_all():
    "Rolls back the Transactions in all registered hubs (for this thread)"
    for hub in hub_registry:
        hub.rollback()

def end_all():
    "Ends the Transactions in all registered hubs (for this thread)"
    for hub in hub_registry:
        hub.end()

[dispatch.generic(MultiorderGenericFunction)]
def run_with_transaction(func, *args, **kw):
    pass

def _use_sa(args=None):
    # check to see if sqlalchemy has been imported and configured
    return _engine is not None

[run_with_transaction.when("not _use_sa(args)")] # include "args" to avoid call being pre-cached
def so_rwt(func, *args, **kw):
    log.debug("Starting SQLObject transaction")
    try:
        try:
            retval = func(*args, **kw)
            commit_all()
            return retval
        except cherrypy.HTTPRedirect:
            commit_all()
            raise
        except cherrypy.InternalRedirect:
            commit_all()
            raise
        except:
            # No need to "rollback" the sqlalchemy unit of work, because nothing
            # has hit the db yet.
            rollback_all()
            raise
    finally:
        end_all()

def dispatch_exception(exception,args, kw):
    # errorhandling import here to avoid circular imports
    from turbogears.errorhandling import dispatch_error
    # Keep in mind func is not the real func but _expose
    real_func, accept, allow_json, controller = args[:4]
    args = args[4:]
    exc_type, exc_value, exc_trace = sys.exc_info()
    remove_keys(kw, ("tg_source", "tg_errors", "tg_exceptions"))
    try:
        output = dispatch_error(
            controller, real_func, None, exception, *args, **kw)
    except dispatch.NoApplicableMethods:
        raise exc_type, exc_value, exc_trace
    else:
        del exc_trace
        return output

[run_with_transaction.when("_use_sa(args)")] # include "args" to avoid call being pre-cached
def sa_rwt(func, *args, **kw):
    log.debug("New SA transaction")
    req = cherrypy.request
    req.sa_transaction = session.create_transaction()
    try:
        retval = func(*args, **kw)
        req.sa_transaction.commit()
    except (cherrypy.HTTPRedirect,cherrypy.InternalRedirect):
        try:
            req.sa_transaction.commit()
        except Exception,e:
            retval = dispatch_exception(e,args,kw)
        else:
            raise
    except Exception, e:
        req.sa_transaction.rollback()
        retval = dispatch_exception(e,args,kw)
    return retval

def so_to_dict(sqlobj):
    "Converts SQLObject to a dictionary based on columns"
    d = {}
    if sqlobj == None:
        # stops recursion
        return d
    for name in sqlobj.sqlmeta.columns.keys():
        d[name] = getattr(sqlobj, name)
    "id must be added explicitly"
    d["id"] = sqlobj.id
    if sqlobj._inheritable:
        d.update(so_to_dict(sqlobj._parent))
        d.pop('childName')
    return d

def so_columns(sqlclass, columns=None):
    """Returns a dict with all columns from a SQLObject including those from
    InheritableSO's bases"""
    if columns is None:
        columns = {}
    columns.update(filter(lambda i: i[0] != 'childName', 
                          sqlclass.sqlmeta.columns.items()))
    if sqlclass._inheritable:
        so_columns(sqlclass.__base__, columns)
    return columns

def so_joins(sqlclass, joins=None):
    """Returns a list with all joins from a SQLObject including those from
    InheritableSO's bases"""
    if joins is None:
        joins = []
    joins.extend(sqlclass.sqlmeta.joins)
    if sqlclass._inheritable:
        so_joins(sqlclass.__base__, joins)
    return joins

class EndTransactionsFilter(BaseFilter):
    def on_end_resource(self):
        if _use_sa():
            session.clear()
        end_all()

__all__ = ["PackageHub", "AutoConnectHub", "set_db_uri",
           "commit_all", "rollback_all", "end_all", "so_to_dict",
           "so_columns", "so_joins", "EndTransactionsFilter"]

if sqlalchemy:
    __all__.extend(["metadata", "session", "bind_meta_data"])



# Formatting configuration for locale is_IS

languages={'is': u'\xedslenska'}

countries={'IS': u'\xcdsland'}

months=[u'jan\xfaar', u'febr\xfaar', 'mars', u'apr\xedl', u'ma\xed', u'j\xfan\xed', u'j\xfal\xed', u'\xe1g\xfast', 'september', u'okt\xf3ber', u'n\xf3vember', 'desember']

abbrMonths=['jan', 'feb', 'mar', 'apr', u'ma\xed', u'j\xfan', u'j\xfal', u'\xe1g\xfa', 'sep', 'okt', u'n\xf3v', 'des']

days=[u'm\xe1nudagur', u'\xferi\xf0judagur', u'mi\xf0vikudagur', 'fimmtudagur', u'f\xf6studagur', 'laugardagur', 'sunnudagur']

abbrDays=[u'm\xe1n', u'\xferi', u'mi\xf0', 'fim', u'f\xf6s', 'lau', 'sun']

dateFormats={'medium': '%d.%m.%Y', 'full': '%%(dayname)s, %d. %%(monthname)s %Y', 'long': '%d. %%(monthname)s %Y', 'short': '%d.%m.%Y'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}


"""Graphical user interface for managing TurboGears projects"""

from turbogears import controllers
from turbogears.util import setlike
from turbogears import identity
import pkg_resources
import turbogears
import cherrypy
import shell
import admi18n
import designer

class Info(controllers.Controller):
    """TurboGears System Information.
       Lists your TurboGears packages and version information
    """
    __label__ ="System Info"
    __version__ = "0.1"
    __author__ = "Ronald Jaramillo"
    __email__ = "ronald@checkandshare.com"
    __copyright__ = "Copyright 2005 Ronald Jaramillo"
    __license__ = "MIT"

    icon = "/tg_static/images/info.png"

    def index(self):
        from turbogears.command.info import retrieve_info
        packages, plugins = retrieve_info()
        return dict(packages=packages, plugins=plugins)
    index = turbogears.expose(template='turbogears.toolbox.info')(index)

class WidgetBrowser(controllers.Controller):
    """Browse usage samples, description and source code for the available TurboGears Widgets"""
    __label__ ="Widget Browser"
    __version__ = "0.1"
    __author__ = "Kevin Dangoor"
    __email__ = "dangoor+turbogears@gmail.com"
    __copyright__ = "Copyright 2005 Kevin Dangoor"
    __license__ = "MIT"
    
    all_descs = None
    icon = "/tg_static/images/widgets.png"
    
    [turbogears.expose(template="turbogears.toolbox.widgets")]
    def index(self, name=None):
        from turbogears import widgets
        from turbogears.widgets import js_location, Tabber, SyntaxHighlighter
        all_descs = self.all_descs
        if not all_descs:
            widgets.load_widgets()
            all_descs = dict()
            for widgetdesc in widgets.all_widgets:
                wd = widgetdesc()
                all_descs[wd.full_class_name.replace(".", "_")] = wd
            self.all_descs = all_descs
        if name:
            all_descs = {name : all_descs[name]}
        desclist = list(all_descs.values())
        desclist.sort(cmp=lambda x,y: cmp(x.name, y.name))
        output = dict(descs=desclist, viewing_one=name != None)
        if name:
            # do not extend desclist!
            desclist = desclist + [Tabber(), SyntaxHighlighter()]
        
        css = setlike()
        js = dict()
        for l in js_location:
            js[l] = setlike()
        
        for widgetdesc in desclist:
            if not name and widgetdesc.show_separately:
                continue
            css.add_all(widgetdesc.retrieve_css())
            
            for l in js_location:
                for script in widgetdesc.retrieve_javascript():
                    if hasattr(script, "location"):
                        js[script.location].add(script)
                    else:
                        js[js_location.head].add(script)
        output["widget_css"] = css
        for l in js_location:
            output["widget_js_%s" % str(l)] = js[l]
        
        return output
    
    def __getattr__(self, widgetname):
        try:
            return self.all_descs[widgetname]
        except:
            raise AttributeError(widgetname)

class Toolbox(controllers.RootController):
    def __init__(self):
        self.toolbox = self.get_tools()

    def tool_icon(self,tool):
        icon = getattr(tool,'icon','')
        if icon: return icon

    def get_tools(self):
        tools = []
        for i in pkg_resources.iter_entry_points("turbogears.toolboxcommand"):

            tool = i.load()
            tools.append((tool,i.name))
            setattr(self, i.name, tool())
        toolbox = []
        for tool, name in tools:
            args = {
                    'path':name,
                    'label':getattr(tool,'__label__',tool),
                    'description':getattr(tool,'__doc__',''),
                    'version':getattr(tool,'__version__',''),
                    'author':getattr(tool,'__author__',''),
                    'email':getattr(tool,'__email__',''),
                    'copyright':getattr(tool,'__copyright__',''),
                    'license':getattr(tool,'__license__',''),
                    'icon':self.tool_icon(tool),
                    'disabled': False
                   }
            if not turbogears.util.get_project_name() and getattr(tool, 'need_project', False):
                args["disabled"] = 'disabled'
            toolbox.append(args)
        return toolbox
    
    def arrange_in_pairs(self,tools):
        p = [[],[]]
        for idx,tool in enumerate(tools): p[idx%2].append(tool)
        pairs = zip(p[0],p[1]) 
        if len(p[0]) > len(p[1]): pairs.append([p[0][-1],None])
        if len(p[0]) < len(p[1]): pairs.append([p[1][-1],None])
        return pairs

    def index(self):
        return dict(toolbox = self.arrange_in_pairs(self.toolbox),
                project = turbogears.util.get_project_name())
    index = turbogears.expose(template="turbogears.toolbox.main")(index)

    def noaccess(self):
        return """<h3>No access for %s</h3>
                    <p>
                       By default only localhost (127.0.0.1) 
                       has access to the Toolbox
                    </p>
                    <p>
                       You can provide access to your client by passing 
                       your host address to Toolbox as a parameter. Ex:
                    </p>
                    <pre>
                        tg-admin toolbox -c %s
                    </pre>
             """% (cherrypy.request.remoteAddr,cherrypy.request.remoteAddr)
    noaccess = turbogears.expose()(noaccess)



import random

from turbojson.jsonify import jsonify_sqlobject, jsonify
from sqlobject import *
from sqlobject.inheritance import InheritableSQLObject

import logging
log = logging.getLogger("turbogears.identity.soprovider")

from datetime import *

import turbogears

from turbogears import identity
from turbogears.database import PackageHub
from turbogears.util import load_class

import warnings

hub = PackageHub("turbogears.identity")
__connection__ = hub

try:
    set, frozenset
except NameError:
    from sets import Set as set, ImmutableSet as frozenset

def to_db_encoding(s, encoding):
    if isinstance(s, str):
        pass
    elif hasattr(s, '__unicode__'):
        s = unicode(s)
    if isinstance(s, unicode):
        s = s.encode(encoding)
    return s

class DeprecatedAttr(object):
    def __init__(self, old_name, new_name):
        self.old_name= old_name
        self.new_name= new_name
    
    def __get__(self, obj, type=None):
        warnings.warn( "%s has been deprecated in favour of %s" %
                       (self.old_name, self.new_name), DeprecationWarning )
        return getattr( obj, self.new_name )

    def __set__(self, obj, value):
        warnings.warn( "%s has been deprecated in favour of %s" %
                       (self.old_name, self.new_name), DeprecationWarning )
        return setattr( obj, self.new_name, value )

        
# Global class references -- these will be set when the Provider is initialised.
user_class= None
group_class= None
permission_class= None
visit_class = None

class SqlObjectIdentity(object):
    def __init__(self, visit_key, user=None):
        if user:
            self._user= user
        self.visit_key= visit_key
    
    def _get_user(self):
        try:
            return self._user
        except AttributeError:
            # User hasn't already been set
            pass
        # Attempt to load the user. After this code executes, there *WILL* be
        # a _user attribute, even if the value is None.
        try:
            visit= visit_class.by_visit_key( self.visit_key )
        except SQLObjectNotFound:
            # The visit ID is invalid
            self._user= None
            return None
        try:
            self._user= user_class.get( visit.user_id )
            return self._user
        except SQLObjectNotFound:
            log.warning( "No such user with ID: %s", visit.user_id )
            self._user= None
            return None
    user= property(_get_user)
    
    def _get_user_name(self):
        if not self.user:
            return None
        return self.user.user_name
    user_name= property(_get_user_name)

    def _get_anonymous(self):
        return not self.user
    anonymous= property(_get_anonymous)
    
    def _get_permissions(self):
        try:
            return self._permissions
        except AttributeError:
            # Permissions haven't been computed yet
            pass
        if not self.user:
            self._permissions= frozenset()
        else:
            self._permissions= frozenset([p.permission_name for p in self.user.permissions])
        return self._permissions
    permissions= property(_get_permissions)
    
    def _get_groups(self):
        try:
            return self._groups
        except AttributeError:
            # Groups haven't been computed yet
            pass
        if not self.user:
            self._groups= frozenset()
        else:
            self._groups= frozenset([g.group_name for g in self.user.groups])
        return self._groups
    groups= property(_get_groups)

    def logout(self):
        '''
        Remove the link between this identity and the visit.
        '''
        try:
            if self.visit_key != None:
                visit= visit_class.by_visit_key(self.visit_key)
                visit.destroySelf()
        except SQLObjectNotFound:
            # If no visit exists in the database, we don't need to destroy it.
            pass
        # Clear the current identity
        anon= SqlObjectIdentity(None,None)
        #XXX if user is None anonymous will be true, no need to set attr.
        #anon.anonymous= True
        identity.set_current_identity( anon )

    
class SqlObjectIdentityProvider(object):
    '''
    IdentityProvider that uses a model from a database (via SQLObject).
    '''
    
    def __init__(self):
        super(SqlObjectIdentityProvider, self).__init__()
        get=turbogears.config.get
        
        global user_class, group_class, permission_class, visit_class
        
        user_class_path= get( "identity.soprovider.model.user", 
                              __name__ + ".TG_User" )
        #log.debug('userclassp:%s'% user_class_path)
        #user_class_path = 'kronos.model.Person'
        user_class= load_class(user_class_path)
        if user_class:
            log.info("Succesfully loaded \"%s\"" % user_class_path)
        try:
            self.user_class_db_encoding= \
                user_class.sqlmeta.columns['user_name'].dbEncoding
        except (KeyError, AttributeError):
            self.user_class_db_encoding= 'UTF-8'
        group_class_path= get( "identity.soprovider.model.group",
                                __name__ + ".TG_Group" )
        group_class= load_class(group_class_path)
        if group_class:
            log.info("Succesfully loaded \"%s\"" % group_class_path)
            
        permission_class_path= get( "identity.soprovider.model.permission",
                                    __name__ + ".TG_Permission" )
        permission_class= load_class(permission_class_path)
        if permission_class:
            log.info("Succesfully loaded \"%s\"" % permission_class_path)
        
        visit_class_path= get( "identity.soprovider.model.visit",
                                __name__ + ".TG_VisitIdentity" )
        visit_class= load_class(visit_class_path)
        if visit_class:
            log.info("Succesfully loaded \"%s\"" % visit_class_path)
        
            
        # Default encryption algorithm is to use plain text passwords
        algorithm = get("identity.soprovider.encryption_algorithm", None)
        self.encrypt_password = lambda pw: \
                                    identity._encrypt_password(algorithm, pw)
            
    def create_provider_model( self ):
        # create the database tables
        try:
            hub.begin()
            user_class.createTable(ifNotExists=True)
            group_class.createTable(ifNotExists=True)
            permission_class.createTable(ifNotExists=True)
            visit_class.createTable(ifNotExists=True)
            hub.commit()
            hub.end()
        except KeyError:
            log.warning( "No database is configured: SqlObjectIdentityProvider is disabled." )
            return

    def validate_identity( self, user_name, password, visit_key ):
        '''
        Look up the identity represented by user_name and determine whether the
        password is correct.
        
        Must return either None if the credentials weren't valid or an object
        with the following properties:
            user_name: original user name
            user: a provider dependant object (TG_User or similar)
            groups: a set of group IDs
            permissions: a set of permission IDs
        '''
        try:
            user_name= to_db_encoding( user_name, self.user_class_db_encoding )
            user= user_class.by_user_name( user_name )
            if not self.validate_password(user, user_name, password):
                log.info( "Passwords don't match for user: %s", user_name )
                return None
            
            # Link the user to the visit
            try:
                link= visit_class.by_visit_key( visit_key )
                link.user_id= user.id
            except SQLObjectNotFound:
                link= visit_class( visit_key=visit_key, user_id=user.id )
            return SqlObjectIdentity( visit_key, user )
            
        except SQLObjectNotFound:
            log.warning( "No such user: %s", user_name )
            return None

    def validate_password(self, user, user_name, password):
        '''
        Check the supplied user_name and password against existing credentials.
        Note: user_name is not used here, but is required by external
        password validation schemes that might override this method.
        If you use SqlObjectIdentityProvider, but want to check the passwords
        against an external source (i.e. PAM, a password file, Windows domain),
        subclass SqlObjectIdentityProvider, and override this method.
        '''
        return user.password == self.encrypt_password(password)

    def load_identity( self, visit_key ):
        '''
        Lookup the principal represented by user_name. Return None if there is no
        principal for the given user ID.
        
        Must return an object with the following properties:
            user_name: original user name
            user: a provider dependant object (TG_User or similar)
            groups: a set of group IDs
            permissions: a set of permission IDs
        '''
        return SqlObjectIdentity( visit_key )
    
    def anonymous_identity( self ):
        '''
        Must return an object with the following properties:
            user_name: original user name
            user: a provider dependant object (TG_User or similar)
            groups: a set of group IDs
            permissions: a set of permission IDs
        '''
        return SqlObjectIdentity( None )

    def authenticated_identity(self, user):
        '''
        Constructs Identity object for user that has no associated visit_key.
        '''
        return SqlObjectIdentity(None, user)


class TG_VisitIdentity(SQLObject):
    class sqlmeta:
        table="tg_visit_identity"

    visit_key= StringCol( length=40, alternateID=True,
                          alternateMethodName="by_visit_key" )
    user_id= IntCol()


class TG_Group(InheritableSQLObject):
    '''
    An ultra-simple group definition.
    '''
    class sqlmeta:
        table="tg_group"
    
    group_name= UnicodeCol( length=16, alternateID=True,
                            alternateMethodName="by_group_name" )
    display_name= UnicodeCol( length=255 )
    created= DateTimeCol( default=datetime.now )

    # Old names
    groupId= DeprecatedAttr( "groupId", "group_name" )
    displayName= DeprecatedAttr( "displayName", "display_name" )

    # collection of all users belonging to this group
    users= RelatedJoin( "TG_User", intermediateTable="tg_user_group",
                        joinColumn="group_id", otherColumn="user_id" )

    # collection of all permissions for this group
    permissions= RelatedJoin( "TG_Permission", joinColumn="group_id", 
                              intermediateTable="tg_group_permission",
                              otherColumn="permission_id" )


def jsonify_group(obj):
    result = jsonify_sqlobject( obj )
    result["users"]= [u.user_name for u in obj.users]
    result["permissions"]= [p.permission_name for p in obj.permissions]
    return result

jsonify_group = jsonify.when('isinstance(obj, TG_Group)')(jsonify_group)


class TG_User(InheritableSQLObject):
    '''
    Reasonably basic User definition. Probably would want additional attributes.
    '''
    class sqlmeta:
        table="tg_user"
    
    user_name= UnicodeCol( length=16, alternateID=True,
                           alternateMethodName="by_user_name" )
    email_address= UnicodeCol( length=255, alternateID=True,
                               alternateMethodName="by_email_address" )
    display_name= UnicodeCol( length=255 )
    password= UnicodeCol( length=40 )
    created= DateTimeCol( default=datetime.now )

    # Old attribute names
    userId= DeprecatedAttr( "userId", "user_name" )
    emailAddress= DeprecatedAttr( "emailAddress", "email_address" )
    displayName= DeprecatedAttr( "displayName", "display_name" )

    # groups this user belongs to
    groups= RelatedJoin( "TG_Group", intermediateTable="tg_user_group",
                         joinColumn="user_id", otherColumn="group_id" )

    def _get_permissions( self ):
        perms= set()
        for g in self.groups:
            perms= perms | set(g.permissions)
        return perms
        
    def _set_password( self, cleartext_password ):
        "Runs cleartext_password through the hash algorithm before saving."
        try:
            hash = identity.current_provider.encrypt_password(cleartext_password)
        except identity.exceptions.IdentityManagementNotEnabledException:
            # Creating identity provider just to encrypt password
            # (so we don't reimplement the encryption step).
            ip = SqlObjectIdentityProvider()
            hash = ip.encrypt_password(cleartext_password)
            if hash == cleartext_password:
                log.info("Identity provider not enabled, and no encryption algorithm "
                        "specified in config.  Setting password as plaintext.")
        self._SO_set_password(hash)
        
    def set_password_raw( self, password ):
        "Saves the password as-is to the database."
        self._SO_set_password(password)


def jsonify_user(obj):
    result = jsonify_sqlobject( obj )
    del result['password']
    result["groups"]= [g.group_name for g in obj.groups]
    result["permissions"]= [p.permission_name for p in obj.permissions]
    return result

jsonify_user = jsonify.when('isinstance(obj, TG_User)')(jsonify_user)


class TG_Permission(InheritableSQLObject):
    class sqlmeta:
        table="tg_permission"
    
    permission_name= UnicodeCol( length=16, alternateID=True,
                                 alternateMethodName="by_permission_name" )
    description= UnicodeCol( length=255 )
    
    # Old attributes
    permissionId= DeprecatedAttr( "permissionId", "permission_name" )

    groups= RelatedJoin( "TG_Group", intermediateTable="tg_group_permission",
                         joinColumn="permission_id", otherColumn="group_id" )


def jsonify_permission(obj):
    result = jsonify_sqlobject( obj )
    result["groups"]= [g.group_name for g in obj.groups]
    return result

jsonify_permission = jsonify.when(
        'isinstance(obj, TG_Permission)')(jsonify_permission)

def encrypt_password(cleartext_password):
    try:
        hash = identity.current_provider.\
                            encrypt_password(cleartext_password)
    except identity.exceptions.RequestRequiredException:
        # Creating identity provider just to encrypt password
        # (so we don't reimplement the encryption step).
        ip = SqlObjectIdentityProvider()
        hash = ip.encrypt_password(cleartext_password)
        if hash == cleartext_password:
            log.info("Identity provider not enabled, and no encryption "
                    "algorithm "
                    "specified in config.  Setting password as plaintext.")
    return hash



import cherrypy

import turbogears
from turbogears import controllers
from turbogears import testutil

class MyRoot(controllers.RootController):
    def set_name(self, name):
        cookies = cherrypy.response.simple_cookie
        cookies['name'] = name
        return "Hello " + name
    set_name = turbogears.expose()(set_name)

    def get_name(self):
        cookies = cherrypy.request.simple_cookie
        if 'name' in cookies:
            return cookies['name'].value
        else:
            return "cookie not found"
    get_name = turbogears.expose()(get_name)
        

def test_browser_session():
    cherrypy.root = MyRoot()
    bs = testutil.BrowsingSession()
    bs.goto('/get_name')
    assert bs.response == 'cookie not found'
    bs.goto('/set_name?name=me')
    bs.goto('/get_name')
    assert bs.response == 'me'

def test_browser_session_for_two_users():
    cherrypy.root = MyRoot()
    bs1 = testutil.BrowsingSession()
    bs2 = testutil.BrowsingSession()
    bs1.goto('/set_name?name=bs1')
    bs2.goto('/set_name?name=bs2')
    bs1.goto('/get_name')
    assert bs1.response == 'bs1'
    bs2.goto('/get_name')
    assert bs2.response == 'bs2'

